import doctest
import pytest

from insights.core.exceptions import SkipComponent, ParseException
from insights.parsers import falconctl
from insights.parsers.falconctl import FalconctlBackend, FalconctlRfm, FalconctlAid, FalconctlVersion
from insights.tests import context_wrap

BACKEND_1 = """
backend is not set.
""".strip()

BACKEND_2 = """
backend=auto.
""".strip()

BACKEND_INVALID_1 = """
backend?auto.
""".strip()

RFM_1 = """
rfm-state=false.
""".strip()

RFM_2 = """
rfm-state=true.
""".strip()

BACKEND_EMPTY = ""
RFM_EMPTY = ""

AID_VALID = """
aid="44e3b7d20b434a2bb2815d9808fa3a8b".
""".strip()

AID_VALID_UNSET = """
aid is not set.
""".strip()

AID_INVALID_1 = """
aid "44e3b7d20b434a2bb2815d9808fa3a8b" .
""".strip()

AID_INVALID_2 = """
aid"44e3b7d20b434a2bb2815d9808fa3a8b".
""".strip()

AID_INVALID_3 = """
aid="44e3b7d20b434a2bb2815d
9808fa3a8b".
""".strip()

VERSION_VALID = """
version = 7.14.16703.0

""".strip()

VERSION_EMPTY = ""

VERSION_INVALID = """
version 7.14.16703.0

""".strip()


def test_falconctl_1():
    backend_1 = FalconctlBackend(context_wrap(BACKEND_1))
    assert backend_1.backend == "not set"


def test_falconctl_2():
    backend_2 = FalconctlBackend(context_wrap(BACKEND_2))
    assert backend_2.backend == "auto"


def test_rfm():
    rfm = FalconctlRfm(context_wrap(RFM_1))
    assert rfm.rfm is False

    rfm = FalconctlRfm(context_wrap(RFM_2))
    assert rfm.rfm is True


def test_backend_empty():
    with pytest.raises(SkipComponent) as e:
        FalconctlBackend(context_wrap(BACKEND_EMPTY))
    assert 'SkipComponent' in str(e)

    with pytest.raises(ParseException) as e:
        FalconctlBackend(context_wrap(BACKEND_INVALID_1))
    assert 'Invalid content:' in str(e)


def test_rfm_empty():
    with pytest.raises(SkipComponent) as e:
        FalconctlRfm(context_wrap(RFM_EMPTY))
    assert 'SkipComponent' in str(e)


def test_falconctl_aid():
    aid = FalconctlAid(context_wrap(AID_VALID))
    assert aid.aid == "44e3b7d20b434a2bb2815d9808fa3a8b"

    aid = FalconctlAid(context_wrap(AID_VALID_UNSET))
    assert aid.aid == "not set"


def test_falconctl_aid_invalid():
    with pytest.raises(SkipComponent) as e:
        FalconctlAid(context_wrap(""))
    assert 'Empty.' in str(e)

    with pytest.raises(ParseException) as e:
        FalconctlAid(context_wrap(AID_INVALID_1))
    assert 'Invalid content:' in str(e)

    with pytest.raises(ParseException) as e:
        FalconctlAid(context_wrap(AID_INVALID_2))
    assert 'Invalid content:' in str(e)

    with pytest.raises(ParseException) as e:
        FalconctlAid(context_wrap(AID_INVALID_2))
    assert 'Invalid content:' in str(e)


def test_falconctl_version():
    falcon_version = FalconctlVersion(context_wrap(VERSION_VALID))
    assert falcon_version.version == "7.14.16703.0"


def test_falconctl_version_invalid():
    with pytest.raises(SkipComponent) as e:
        FalconctlVersion(context_wrap(VERSION_EMPTY))
    assert 'Empty.' in str(e)

    with pytest.raises(ParseException) as e:
        FalconctlVersion(context_wrap(VERSION_INVALID))
    assert 'Invalid content:' in str(e)


def test_falcontcl_doc_examples():
    env = {
        "falconctlbackend": FalconctlBackend(context_wrap(BACKEND_2)),
        "falconctlrfm": FalconctlRfm(context_wrap(RFM_1)),
        "falconctlaid": FalconctlAid(context_wrap(AID_VALID)),
        "falconctlversion": FalconctlVersion(context_wrap(VERSION_VALID))
    }
    failed, total = doctest.testmod(falconctl, globs=env)
    assert failed == 0



from insights.parsers.heat_log import HeatApiLog
from insights.parsers.heat_log import HeatEngineLog
from insights.tests import context_wrap

from datetime import datetime


HEAT_API_LOG = """
2016-11-09 14:39:29.223 3844 WARNING oslo_config.cfg [-] Option "rpc_backend" from group "DEFAULT" is deprecated for removal.  Its value may be silently ignored in the future.
2016-11-09 14:39:30.612 3844 INFO heat.api [-] Starting Heat REST API on 172.16.2.12:8004
2016-11-09 14:39:30.612 3844 WARNING oslo_reports.guru_meditation_report [-] Guru meditation now registers SIGUSR1 and SIGUSR2 by default for backward compatibility. SIGUSR1 will no longer be registered in a future release, so please use SIGUSR2 to generate reports.
2016-11-09 14:39:30.615 3844 INFO heat.common.wsgi [-] Starting 0 workers
2016-11-09 14:39:30.625 3844 INFO heat.common.wsgi [-] Started child 4136
"""


HEAT_ENGINE_LOG = """
2016-11-09 14:32:44.157 4392 WARNING heat.engine.environment [-] OS::Neutron::PoolMember is DEPRECATED. Neutron LBaaS v1 is deprecated in the Liberty release and is planned to be removed in a future release. Going forward, the LBaaS V2 should be used.
2016-11-09 14:32:44.157 4392 WARNING heat.engine.environment [-] OS::Neutron::HealthMonitor is DEPRECATED. Neutron LBaaS v1 is deprecated in the Liberty release and is planned to be removed in a future release. Going forward, the LBaaS V2 should be used.
2016-11-09 14:32:44.158 4392 WARNING heat.engine.environment [-] OS::Neutron::RouterGateway is HIDDEN. Use the `external_gateway_info` property in the router resource to set up the gateway.
2016-11-09 14:32:44.160 4392 INFO heat.engine.environment [-] Loading /etc/heat/environment.d/default.yaml
2016-11-09 14:32:44.168 4392 INFO heat.engine.environment [-] Registered: [Plugin](User:False) AWS::EC2::Instance -> <class 'heat.engine.resources.aws.ec2.instance.Instance'>
2016-11-09 14:32:45.169 4392 INFO heat.engine.environment [-] Registered: [Plugin](User:False) OS::Heat::AccessPolicy -> <class 'heat.engine.resources.openstack.heat.access_policy.AccessPolicy'>
"""


def test_heat_api_log():
    log = HeatApiLog(context_wrap(HEAT_API_LOG))
    assert len(log.get('INFO')) == 3
    assert len(list(log.get_after(datetime(2016, 11, 9, 14, 39, 30)))) == 4


def test_heat_engine_log():
    log = HeatEngineLog(context_wrap(HEAT_ENGINE_LOG))
    assert len(log.get('INFO')) == 3
    assert len(list(log.get_after(datetime(2016, 11, 9, 14, 32, 45)))) == 1



from insights.parsers import cloud_init_custom_network
from insights.parsers.cloud_init_custom_network import CloudInitCustomNetworking
from insights.tests import context_wrap
import doctest

CLOUD_INIT_CONFIG = """
network:
    version: 1
    config:
    - type: physical
      name: eth0
      subnets:
        - type: dhcp
        - type: dhcp6
"""


def test_cloud_init_custom_network():
    result = cloud_init_custom_network.CloudInitCustomNetworking(context_wrap(CLOUD_INIT_CONFIG))
    assert result.data['network']['config'][0]['name'] == 'eth0'
    assert result.data['network']['config'][0]['subnets'][0]['type'] == 'dhcp'
    assert result.data['network']['config'][0]['subnets'][1]['type'] == 'dhcp6'


def test_cloud_init_custom_networks_doc_examples():
    env = {
        'cloud_init_custom_network_config': CloudInitCustomNetworking(context_wrap(CLOUD_INIT_CONFIG)),
    }
    failed, total = doctest.testmod(cloud_init_custom_network, globs=env)
    assert failed == 0



# -*- coding: UTF-8 -*-

from insights.client.constants import InsightsConstants
from insights.client.phase.v1 import pre_update
from mock.mock import patch
from pytest import raises


def patch_insights_config(old_function):
    patcher = patch("insights.client.phase.v1.InsightsConfig",
                    **{"return_value.load_all.return_value.auto_config": False,
                       "return_value.load_all.return_value.version": False,
                       "return_value.load_all.return_value.validate": False,
                       "return_value.load_all.return_value.enable_schedule": False,
                       "return_value.load_all.return_value.disable_schedule": False,
                       "return_value.load_all.return_value.analyze_container": False,
                       "return_value.load_all.return_value.test_connection": False,
                       "return_value.load_all.return_value.support": False,
                       "return_value.load_all.return_value.diagnosis": False,
                       "return_value.load_all.return_value.checkin": True})
    return patcher(old_function)


@patch("insights.client.phase.v1.InsightsClient", **{"return_value.checkin.return_value": True})
@patch_insights_config
def test_checkin_success(insights_config, insights_client):
    """
    InsightsSupport is constructed with InsightsConfig and collect_support_info is called.
    """
    with raises(SystemExit) as exc_info:
        pre_update()

    insights_client.return_value.checkin.assert_called_once_with()
    assert exc_info.value.code == InsightsConstants.sig_kill_ok


@patch("insights.client.phase.v1.InsightsClient", **{"return_value.checkin.return_value": False})
@patch_insights_config
def test_checkin_failure(insights_config, insights_client):
    """
    Support collection replaces the normal client run.
    """
    with raises(SystemExit) as exc_info:
        pre_update()

    insights_client.return_value.checkin.assert_called_once_with()
    assert exc_info.value.code == InsightsConstants.sig_kill_bad



import copy
import os
import six
import tempfile

from glob import glob
from logging import getLogger
from re import findall
from sys import exit

from insights.client.auto_config import try_auto_configuration
from insights.client.connection import InsightsConnection
from insights.client.constants import InsightsConstants as constants
from insights.util.subproc import call


NONCOMPLIANT_STATUS = 2
OUT_OF_MEMORY_STATUS = -9  # 247
COMPLIANCE_CONTENT_TYPE = 'application/vnd.redhat.compliance.something+tgz'
POLICY_FILE_LOCATION = '/usr/share/xml/scap/ssg/content/'
SCAP_DATASTREAMS_PATH = '/usr/share/xml/scap/'
SSG_PACKAGE = 'scap-security-guide'
REQUIRED_PACKAGES = [SSG_PACKAGE, 'openscap-scanner', 'openscap']
OOM_ERROR_LINK = 'https://access.redhat.com/articles/6999111'

# SSG versions that need the <version> in XML repaired
VERSIONS_FOR_REPAIR = '0.1.18 0.1.19 0.1.21 0.1.25'.split()
SNIPPET_TO_FIX = '<version>0.9</version>'

logger = getLogger(__name__)


class ComplianceClient:
    def __init__(self, os_version=None, ssg_version=None, config=None):
        self._inventory_id = None
        self.os_major, self.os_minor = os_version if os_version else [None, None]
        self.ssg_version = ssg_version
        config = copy.deepcopy(config)
        if config.legacy_upload is True:
            # For compliance, use "legacy_upload=False" by force
            config.legacy_upload = False
            try_auto_configuration(config)
        self.conn = InsightsConnection(config)

    def fetch_tailoring_content(self, policy):
        if 'os_minor_version' in policy and policy['os_minor_version'] != self.os_minor:
            return None

        logger.debug(
            "Checking if policy {0} is a tailored policy. Starting tailoring file download...".format(
                policy['ref_id']
            )
        )
        response = self.conn.session.get(
            "{0}/compliance/v2/policies/{1}/tailorings/{2}/tailoring_file".format(
                self.conn.base_url, policy['id'], self.os_minor
            )
        )
        logger.debug("Response code: {0}".format(response.status_code))

        if response.status_code == 204:
            logger.debug(
                "Policy {0} is not tailored, continuing with default rule and value selections...".format(
                    policy['ref_id']
                )
            )
            return None

        if not response.ok:
            logger.debug(
                "Something went wrong during downloading the tailoring file of {0}. The expected status code is 200, got {1}".format(
                    policy['ref_id'], response.status_code
                )
            )
            return None

        if response.content is None or response.headers['Content-Type'] != "application/xml":
            logger.debug(response.content)
            logger.debug(response.return_value)
            logger.debug(
                "Problem with the content of the downloaded tailoring file of {0}. The expected format is xml, got {1}".format(
                    policy['ref_id'], response.headers['Content-Type']
                )
            )
            return None

        return response.content

    def download_tailoring_file(self, policy):
        # Download tailoring file to pass as argument to run_scan
        content = self.fetch_tailoring_content(policy)
        # Check if the content is empty, and if so, don't create the file at all
        if content is None or not content.strip():
            logger.debug(
                "The tailoring file for policy {0} is empty. Skipping file creation.".format(
                    policy['ref_id']
                )
            )
            return None

        # Create a temporary file if content is valid (not empty)
        tailoring_file_path = tempfile.mkstemp(
            prefix='oscap_tailoring_file-{0}.'.format(policy['ref_id']),
            suffix='.xml',
            dir='/var/tmp',
        )[1]

        with open(tailoring_file_path, mode="w+b") as f:
            f.write(content)
            logger.info(
                "Saved tailoring file for {0} to {1}".format(policy['ref_id'], tailoring_file_path)
            )

        logger.debug("Policy {0} tailoring file download finished".format(policy['ref_id']))

        return tailoring_file_path

    def profile_files(self):
        return glob("{0}*rhel{1}-ds.xml".format(POLICY_FILE_LOCATION, self.os_major))

    def find_scap_policy(self, profile_ref_id):
        grepcmd = 'grep -H ' + profile_ref_id + ' ' + ' '.join(self.profile_files())
        if not six.PY3:
            grepcmd = grepcmd.encode()
        rc, grep = call(grepcmd, keep_rc=True)
        if rc:
            logger.error(
                'XML profile file not found matching ref_id {0}\n{1}\n'.format(profile_ref_id, grep)
            )
            return None
        filenames = findall(SCAP_DATASTREAMS_PATH + '.+xml', grep)
        if not filenames:
            logger.error(
                'No XML profile files found matching ref_id {0}\n{1}\n'.format(
                    profile_ref_id, ' '.join(filenames)
                )
            )
            exit(constants.sig_kill_bad)
        return filenames[0]

    def build_oscap_command(self, profile_ref_id, policy_xml, output_path, tailoring_file_path):
        command = 'oscap xccdf eval --profile ' + profile_ref_id
        if tailoring_file_path:
            command += ' --tailoring-file ' + tailoring_file_path
        command += ' --results ' + output_path + ' ' + policy_xml
        return command

    def run_scan(self, profile_ref_id, policy_xml, output_path, tailoring_file_path=None):
        if policy_xml is None:
            return
        logger.info('Running scan for {0}... this may take a while'.format(profile_ref_id))
        env = os.environ.copy()
        env.update({'TZ': 'UTC'})
        oscap_command = self.build_oscap_command(
            profile_ref_id, policy_xml, output_path, tailoring_file_path
        )
        if not six.PY3:
            oscap_command = oscap_command.encode()
        rc, oscap = call(oscap_command, keep_rc=True, env=env)

        if rc and rc == OUT_OF_MEMORY_STATUS:
            logger.error('Scan failed due to insufficient memory')
            logger.error('More information can be found here: {0}'.format(OOM_ERROR_LINK))
            exit(constants.sig_kill_bad)

        if rc and rc != NONCOMPLIANT_STATUS:
            logger.error('Scan failed')
            logger.error(rc)
            logger.error(oscap)
            exit(constants.sig_kill_bad)

    # Helper function that traverses through the XCCDF report and replaces the content of each
    # matching xpath with an empty string
    def obfuscate(self, tree, xpaths):
        for xpath in xpaths:
            for node in tree.findall(xpath):
                node.text = ''

    def results_need_repair(self):
        return self.ssg_version in VERSIONS_FOR_REPAIR

    def repair_results(self, results_file, content):
        if not self.ssg_version:
            logger.warning("Couldn't repair SSG version in results file %s", results_file)
            return

        replacement = '<version>{0}</version>'.format(self.ssg_version)
        is_repaired = False
        for idx, line in enumerate(content):
            if not (is_repaired or SNIPPET_TO_FIX not in line):
                content[idx] = line.replace(SNIPPET_TO_FIX, replacement)
                is_repaired = True
                logger.debug(
                    'Substituted "%s" with "%s" in %s', SNIPPET_TO_FIX, replacement, results_file
                )

        if is_repaired:
            logger.debug('Repaired version in results file %s', results_file)

        return content

    @property
    def inventory_id(self):
        if self._inventory_id:
            return self._inventory_id

        system = self.conn._fetch_system_by_machine_id()
        if isinstance(system, dict) and 'id' in system:
            self._inventory_id = system.get('id')
            return self._inventory_id
        else:
            logger.debug('Failed to find system in Inventory')

    def assignable_policies(self):
        url = "{0}/compliance/v2/policies?filter=(os_major_version={1} and os_minor_version={2})&limit=100"
        full_url = url.format(self.conn.base_url, self.os_major, self.os_minor)
        logger.debug("Fetching policies with: {0}".format(full_url))
        response = self.conn.session.get(full_url)
        logger.debug("Content of the response {0} - {1}".format(response, response.content))
        assigned_policies = [item["id"] for item in self.get_system_policies()]

        if response.status_code == 200:
            policies = response.json().get('data', [])
            if not policies:
                logger.warning(
                    "System is not assignable to any policy. Create supported policy using the Compliance web UI.\n"
                )
                return constants.sig_kill_bad
            else:
                print("Assigned     ID" + " " * 39 + "Title")
                for policy in policies:
                    is_assigned = policy['id'] in assigned_policies
                    print("%-12s %-40s %s" % (is_assigned, policy['id'], policy['title']))
            return 0
        else:
            logger.error("An error has occurred while communicating with the API.\n")
            return constants.sig_kill_bad

    def policy_link(self, policy_id, opt):
        if self.inventory_id is None:
            # return SIGKIL directly when inventory_id cannot be found
            logger.error('Failed to find system in Inventory')
            return constants.sig_kill_bad
        url = "{0}/compliance/v2/policies/{1}/systems/{2}"
        full_url = url.format(self.conn.base_url, policy_id, self.inventory_id)
        logger.debug("Fetching: {0}".format(full_url))
        response = getattr(self.conn.session, opt)(full_url)
        logger.debug("Content of the response {0} - {1}".format(response, response.content))

        operation = "unassigned" if opt == "delete" else "assigned"
        if response.status_code == 202:
            logger.info("Successfully {0} policy (ID {1}).\n".format(operation, policy_id))
            return 0
        else:
            logger.error(
                "Policy ID {0} can not be {1}. "
                "Refer to the /var/log/insights-client/insights-client.log for more details.".format(
                    policy_id,
                    operation,
                )
            )
            return constants.sig_kill_bad

    def get_system_policies(self):
        if self.inventory_id is None:
            # return empty directly when inventory_id cannot be found
            return []
        url = "{0}/compliance/v2/systems/{1}/policies".format(self.conn.base_url, self.inventory_id)
        logger.debug("Fetching policies with: {0}".format(url))
        response = self.conn.session.get(url)
        logger.debug("Content of the response {0} - {1}".format(response, response.content))

        if response.status_code == 200:
            return response.json().get('data', [])
        else:
            return []



"""
1. The rule uses one Combiner that depends upon a filterable Parser.
2. The rule does also depend upon (import) the filterable Parser directly.
3. The filterable Parser is FILTERED in the rule.

=> Exception is NOT expected
"""
from insights import rule, make_pass
from insights.combiners.os_release import OSRelease
from insights.core.filters import add_filter
from insights.parsers.dmesg import DmesgLineList

add_filter(DmesgLineList, 'Linux version')


@rule(OSRelease, DmesgLineList)
def parser_is_filtered(osr, dmesg):
    return make_pass("FAKE RESULT")



"""
Container logs
==============

Module for parsing the log files for Openshift kubelet container logs.


ControllerManagerLog - file ``/var/log/pods/openshift-kube-controller-manager*/*/*.log``
----------------------------------------------------------------------------------------

ApiServerLog - file ``/var/log/pods/openshift-kube-apiserver*/*/*.log``
-----------------------------------------------------------------------

.. note::
    Please refer to the super-class :class:`insights.core.LogFileOutput`
"""

from insights import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.controller_manager_log)
class ControllerManagerLog(LogFileOutput):
    """Class for parsing ``/var/log/pods/openshift-kube-controller-manager*/*/*.log`` file.

    Typical content of ``*.log`` file is::

        2023-12-18T06:13:36.831359143+00:00 stderr F I1218 06:13:36.831274      16 aggregator.go:115] Building initial OpenAPI spec
        2023-12-18T06:13:36.846052152+00:00 stderr F I1218 06:13:36.845970      16 aggregator.go:118] Finished initial OpenAPI spec generation after 14.671324ms
        2024-12-18T06:13:37.031688793+00:00 stderr F I1218 06:13:37.031622      16 genericapiserver.go:535] "[graceful-termination] using HTTP Server shutdown timeout" ShutdownTimeout="2s"
        2024-12-18T06:13:37.031809098+00:00 stderr F I1218 06:13:37.031766      16 dynamic_cafile_content.go:157] "Starting controller" name="client-ca-bundle::/etc/kubernetes/static-pod-certs/configmaps/client-ca/ca-bundle.crt"
        2023-12-18T06:13:37.031901208+00:00 stderr F I1218 06:13:37.031766      16 dynamic_cafile_content.go:157] "Starting controller" name="request-header::/etc/kubernetes/static-pod-certs/configmaps/aggregator-client-ca/ca-bundle.crt"
    """
    time_format = '%Y-%m-%dT%H:%M:%S.%f'


@parser(Specs.api_server_log)
class ApiServerLog(LogFileOutput):
    """Class for parsing ``var/log/pods/openshift-kube-apiserver*/*/*.log`` file.

    Typical content of ``*.log`` file is::

        2023-12-18T06:13:36.831359143+00:00 stderr F I1218 06:13:36.831274      16 aggregator.go:115] Building initial OpenAPI spec
        2023-12-18T06:13:36.846052152+00:00 stderr F I1218 06:13:36.845970      16 aggregator.go:118] Finished initial OpenAPI spec generation after 14.671324ms
        2024-12-18T06:13:37.031688793+00:00 stderr F I1218 06:13:37.031622      16 genericapiserver.go:535] "[graceful-termination] using HTTP Server shutdown timeout" ShutdownTimeout="2s"
        2024-12-18T06:13:37.031809098+00:00 stderr F I1218 06:13:37.031766      16 dynamic_cafile_content.go:157] "Starting controller" name="client-ca-bundle::/etc/kubernetes/static-pod-certs/configmaps/client-ca/ca-bundle.crt"
        2023-12-18T06:13:37.031901208+00:00 stderr F I1218 06:13:37.031766      16 dynamic_cafile_content.go:157] "Starting controller" name="request-header::/etc/kubernetes/static-pod-certs/configmaps/aggregator-client-ca/ca-bundle.crt"
    """
    time_format = '%Y-%m-%dT%H:%M:%S.%f'



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import mac
from insights.tests import context_wrap


MAC_OK = """
00:5b:ea:09:00:00
""".strip()

MAC_NG = """
h0:5b:ea:09:00:00
""".strip()


def test_mac_address():
    ret = mac.MacAddress(context_wrap(MAC_OK))
    assert ret.address == '00:5b:ea:09:00:00'

    with pytest.raises(SkipComponent):
        mac.MacAddress(context_wrap(MAC_NG))


def test_mac_doc():
    env = {
            'mac_addr': mac.MacAddress(context_wrap(MAC_OK)),
          }
    failed, total = doctest.testmod(mac, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import kernel_config
from insights.parsers.kernel_config import KernelConf
from insights.tests import context_wrap

KERNEL_CONFIG = """
#
# Automatically generated file; DO NOT EDIT.
# Linux/x86_64 3.10.0-693.el7.x86_64 Kernel Configuration
#
CONFIG_64BIT=y
CONFIG_X86_64=y
CONFIG_X86=y
CONFIG_INSTRUCTION_DECODER=y
CONFIG_OUTPUT_FORMAT="elf64-x86-64"
CONFIG_ARCH_DEFCONFIG="arch/x86/configs/x86_64_defconfig"
CONFIG_ARCH_MMAP_RND_COMPAT_BITS_MIN=8
CONFIG_PREEMPT_RT_FULL=y
""".strip()

KERNEL_CONFIG_2 = """
#
# Automatically generated file; DO NOT EDIT.
# Linux/x86_64 3.10.0-693.el7.x86_64 Kernel Configuration
#
CONFIG_64BIT=y
CONFIG_X86_64=y
CONFIG_X86=y
CONFIG_INSTRUCTION_DECODER=y
CONFIG_OUTPUT_FORMAT="elf64-x86-64"
CONFIG_ARCH_DEFCONFIG="arch/x86/configs/x86_64_defconfig"
CONFIG_ARCH_MMAP_RND_COMPAT_BITS_MIN=8
# CONFIG_PREEMPT_RT_FULL is not set
# CONFIG_IRQ_DOMAIN_DEBUG is not set
""".strip()

KCONFIG_FILE_PATH = "/boot/config-3.10.0-327.28.3.rt56.235.el7.x86_64"

KERNEL_CONFIG_NO = """
""".strip()

KERNEL_CONFIG_NO_2 = """
#
# Automatically generated file; DO NOT EDIT.
# Linux/x86_64 3.10.0-693.el7.x86_64 Kernel Configuration
#
""".strip()

KCONFIG_FILE_PATH_NO = ""


def test_kernel_config():
    r = KernelConf(context_wrap(KERNEL_CONFIG, KCONFIG_FILE_PATH))
    assert r.get("CONFIG_PREEMPT_RT_FULL") == "y"
    assert len(r) == 8
    assert r.kconf_file == "config-3.10.0-327.28.3.rt56.235.el7.x86_64"

    r = KernelConf(context_wrap(KERNEL_CONFIG_2, KCONFIG_FILE_PATH))
    assert len(r) == 7

    with pytest.raises(SkipComponent) as exc:
        r = KernelConf(context_wrap(KERNEL_CONFIG_NO, KCONFIG_FILE_PATH))
    assert 'No Contents' in str(exc)


def test_docs():
    env = {
        'kconfig': KernelConf(context_wrap(KERNEL_CONFIG, KCONFIG_FILE_PATH))
    }
    failed, total = doctest.testmod(kernel_config, globs=env)
    assert failed == 0



import sys
from insights import get_nvr
from insights.command_parser import InsightsCli
import pytest
from mock import patch

VERSION_OUT = get_nvr() + '\n'


def test_insights_cli_version(capsys):
    test_args_list = [
        ["insights", "version"],
        ["insights", "--version"],
        ["insights", "cat", "--version"],
        ["insights", "collect", "--version"],
        ["insights", "inspect", "--version"],
        ["insights", "info", "--version"],
        ["insights", "ocpshell", "--version"],
        ["insights", "run", "--version"]
    ]
    for args in test_args_list:
        with patch.object(sys, 'argv', args):
            with pytest.raises(SystemExit):
                InsightsCli()
            out, err = capsys.readouterr()
            assert err == ''
            assert out == VERSION_OUT


def test_insights_cli_version_bad_arg(capsys):
    test_args = ["insights", "-v"]
    with patch.object(sys, 'argv', test_args):
        with pytest.raises(SystemExit):
            InsightsCli()
        out, err = capsys.readouterr()
        assert len(err) > 0
        assert out == ''



import pytest

from mock.mock import patch
from insights.core.exceptions import SkipComponent
from insights.parsers.iris import IrisList, IrisCpf
from insights.specs.datasources.intersystems import iris_working_configuration, iris_working_messages_log
from insights.tests import context_wrap


IRIR_LIST = """

Configuration 'IRIS'   (default)
    directory:    /intersystems
    versionid:    2023.1.0.235.1com
    datadir:      /intersystems
    conf file:    iris.cpf  (SuperServer port = 1972, WebServer = 52773)
    status:       running, since Tue Sep 12 09:49:54 2023
    state:        ok
    product:      InterSystems IRIS

Configuration 'IRIS2'
    directory:    /intersystems2
    versionid:    2023.1.0.235.1com
    datadir:      /intersystems2
    conf file:    iris.cpf  (SuperServer port = 51773, WebServer = 52774)
    status:       running, since Tue Sep 12 09:50:05 2023
    state:        ok
    product:      InterSystems IRIS
""".strip()


IRIS_CPF = """
[ConfigFile]
Product=IRIS
Version=2023.1

[Databases]
IRISSYS=/intersystems/mgr/
IRISLIB=/intersystems/mgr/irislib/
IRISTEMP=/intersystems/mgr/iristemp/
IRISLOCALDATA=/intersystems/mgr/irislocaldata/
IRISAUDIT=/intersystems/mgr/irisaudit/
ENSLIB=/intersystems/mgr/enslib/
USER=/intersystems/mgr/user/

[Namespaces]
%SYS=IRISSYS
USER=USER

[config]
bbsiz=-1
console=,
""".strip()

IRIS_CPF_2 = """
[ConfigFile]
Product=IRIS2
Version=2023.1

[Databases]
IRISSYS=/intersystems2/mgr/
IRISLIB=/intersystems/mgr2/irislib/
IRISTEMP=/intersystems/mgr2/iristemp/
IRISLOCALDATA=/intersystems2/mgr/irislocaldata/
IRISAUDIT=/intersystems2/mgr/irisaudit/
ENSLIB=/intersystems2/mgr/enslib/
USER=/intersystems2/mgr/user/

[Namespaces]
%SYS=IRISSYS
USER=USER

[config]
bbsiz=-1
console=,
""".strip()


RAW_MESSAGES = """
06/26/23-08:02:17:828 (144145) 0 [Generic.Event] Allocated 495MB shared memory
06/26/23-08:02:17:828 (144145) 0 [Generic.Event] 32MB global buffers, 80MB routine buffers, 64MB journal buffers, 4MB buffer descriptors, 300MB heap, 5MB ECP, 9MB miscellaneous
06/26/23-08:02:17:831 (144145) 0 [Crypto.IntelSandyBridgeAESNI] Intel Sandy Bridge AES-NI instructions detected.
06/26/23-08:02:17:831 (144145) 0 [SIMD] SIMD optimization level: DEFAULT X86
06/26/23-08:02:17:903 (144145) 0 [WriteDaemon.UsingWIJFile] Using WIJ file: /intersystems/mgr/IRIS.WIJ
06/26/23-08:02:17:903 (144145) 0 [Generic.Event] No journaling info from prior system
06/26/23-08:02:17:903 (144145) 0 [WriteDaemon.CreatingNewWIJ] Creating a new WIJ file
06/26/23-08:02:18:104 (144145) 0 [WriteDaemon.CreatedNewWIJ] New WIJ file created
06/26/23-08:02:18:110 (144145) 0 [Generic.Event]
""".strip()


FILTERED_MESSAGES = """
06/26/23-08:02:17:828 (144145) 0 [Generic.Event] Allocated 495MB shared memory
06/26/23-08:02:17:828 (144145) 0 [Generic.Event] 32MB global buffers, 80MB routine buffers, 64MB journal buffers, 4MB buffer descriptors, 300MB heap, 5MB ECP, 9MB miscellaneous
06/26/23-08:02:17:903 (144145) 0 [Generic.Event] No journaling info from prior system
06/26/23-08:02:18:110 (144145) 0 [Generic.Event]
""".strip()


@patch("os.path.isfile", return_value=True)
def test_iris_working_configuration(m_isfile):
    iris_list_info = IrisList(context_wrap(IRIR_LIST))
    broker = {IrisList: iris_list_info}
    result = iris_working_configuration(broker)
    assert set(result) == set(["/intersystems/iris.cpf", "/intersystems2/iris.cpf"])


def test_iris_working_configuration_no_file():
    iris_list_info = IrisList(context_wrap(IRIR_LIST))
    broker = {IrisList: iris_list_info}
    with pytest.raises(SkipComponent) as e:
        iris_working_configuration(broker)
    assert 'SkipComponent' in str(e)


@patch("os.path.isfile", return_value=True)
def test_iris_working_messages_log(m_isfile):
    iris_cpf_info = IrisCpf(context_wrap(IRIS_CPF))
    iris_cpf_info_2 = IrisCpf(context_wrap(IRIS_CPF_2))
    broker = {IrisCpf: [iris_cpf_info, iris_cpf_info_2]}
    result = iris_working_messages_log(broker)
    assert set(result) == set(['/intersystems/mgr/messages.log', '/intersystems2/mgr/messages.log'])


def test_iris_working_messages_log_no_file():
    iris_cpf_info = IrisCpf(context_wrap(IRIS_CPF))
    iris_cpf_info_2 = IrisCpf(context_wrap(IRIS_CPF_2))
    broker = {IrisCpf: [iris_cpf_info, iris_cpf_info_2]}
    with pytest.raises(SkipComponent) as e:
        iris_working_messages_log(broker)
    assert 'SkipComponent' in str(e)



import pytest

from mock.mock import patch, PropertyMock

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent, ContentException
from insights.core.spec_factory import ContainerCommandProvider
from insights.specs.datasources.container.nginx_conf import LocalSpecs, nginx_conf

CONTAINER_CMD = '%s exec -e "PATH=$PATH" %s sh -c "command -v find && find /etc/ /opt/ *.conf"'

find_list = [
    '/etc/nginx/nginx.conf',
    '/etc/nginx/nginx.conf.d/ss-nginx.conf',
    '/opt/rh-nginx/nginx.conf',
    '/opt/rh-nginx/nginx.conf.d/ss-nginx.conf',
]

find_list_ng = [
    '/etc/rd-nginx/nginx.conf',
    '/etc/rd-nginx/nginx.conf.d/ss-nginx.conf',
    '/opt/nginx/nginx.conf',
    '/opt/nginx/nginx.conf.d/ss-nginx.conf',
]

# with spec keep_rc, cmd CalledProcessError would be kept as self._content
find_cmd_exec_error = [
    'Error: runc: exec failed: unable to start container process: exec: '
    + '"find": executable file not found in $PATH: OCI runtime attempted to'
    + 'invoke a command that was not found'
]

find_result_empty = ContentException(
    "Empty after cleaning: insights_containers/03e2861336a6/insights_commands/find_.etc_.opt_-name_.conf"
)


@patch("insights.core.spec_factory.CommandOutputProvider.validate", return_value=None)
@patch("insights.core.spec_factory.CommandOutputProvider.load", return_value=None)
@patch(
    "insights.core.spec_factory.CommandOutputProvider.content",
    new_callable=PropertyMock,
    return_value=find_list,
)
def test_nginx_conf(validate, load, find_l):
    find_nginx_confs = [
        ContainerCommandProvider(CONTAINER_CMD % ("podman", "03e2861336a7"), None, 'rhel8_nginx'),
        ContainerCommandProvider(CONTAINER_CMD % ("docker", "03e2861336a8"), None, 'rhel7_nginx'),
    ]
    broker = {LocalSpecs.container_find_etc_opt_conf: find_nginx_confs, HostContext: None}

    ret = nginx_conf(broker)
    assert len(ret) == 8
    assert ('rhel8_nginx', 'podman', '03e2861336a7', '/etc/nginx/nginx.conf') in ret
    assert (
        'rhel7_nginx',
        'docker',
        '03e2861336a8',
        '/opt/rh-nginx/nginx.conf.d/ss-nginx.conf',
    ) in ret


@patch("insights.core.spec_factory.CommandOutputProvider.validate", return_value=None)
@patch("insights.core.spec_factory.CommandOutputProvider.load", return_value=None)
@patch(
    "insights.core.spec_factory.CommandOutputProvider.content",
    new_callable=PropertyMock,
    return_value=find_list_ng,
)
def test_nginx_conf_empty(validate, load, find_l):
    find_nginx_confs_ng = [
        ContainerCommandProvider(CONTAINER_CMD % ("podman", "03e2861336a7"), None, 'rhel8_nginx'),
        ContainerCommandProvider(CONTAINER_CMD % ("docker", "03e2861336a8"), None, 'rhel7_nginx'),
    ]
    broker = {LocalSpecs.container_find_etc_opt_conf: find_nginx_confs_ng, HostContext: None}

    with pytest.raises(SkipComponent):
        ret = nginx_conf(broker)
        assert len(ret) == 0


@patch("insights.core.spec_factory.CommandOutputProvider.validate", return_value=None)
@patch("insights.core.spec_factory.CommandOutputProvider.load", return_value=None)
@patch(
    "insights.core.spec_factory.CommandOutputProvider.content",
    new_callable=PropertyMock,
    side_effect=[find_cmd_exec_error, find_result_empty, find_list_ng, find_list],
)
def test_nginx_conf_on_cmd_error(validate, load, find_l):
    find_nginx_confs = [
        ContainerCommandProvider(CONTAINER_CMD % ("podman", "03e2861336a5"), None, 'rhel8_nginx'),
        ContainerCommandProvider(CONTAINER_CMD % ("docker", "03e2861336a6"), None, 'rhel7_nginx'),
        ContainerCommandProvider(CONTAINER_CMD % ("podman", "03e2861336a7"), None, 'rhel8_nginx'),
        ContainerCommandProvider(CONTAINER_CMD % ("docker", "03e2861336a8"), None, 'rhel7_nginx'),
    ]
    broker = {LocalSpecs.container_find_etc_opt_conf: find_nginx_confs, HostContext: None}

    ret = nginx_conf(broker)
    assert len(ret) == 4
    assert ('rhel8_nginx', 'podman', '03e2861336a5', '/etc/nginx/nginx.conf') not in ret
    assert (
        'rhel7_nginx',
        'docker',
        '03e2861336a6',
        '/opt/rh-nginx/nginx.conf.d/ss-nginx.conf',
    ) not in ret
    assert ('rhel8_nginx', 'podman', '03e2861336a7', '/etc/nginx/nginx.conf') not in ret
    assert (
        'rhel7_nginx',
        'docker',
        '03e2861336a8',
        '/opt/rh-nginx/nginx.conf.d/ss-nginx.conf',
    ) in ret



"""
CephVersion - command ``ceph -v``
=================================

This module provides plugins access to the Ceph version information gathered from
the ``ceph -v`` command. This module parses the community version to the Red Hat
release version.

The Red Hat Ceph Storage releases and corresponding Ceph package releases are
documented in https://access.redhat.com/solutions/2045583

"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs
from insights.util import rsplit

# TODO: the following metrics need update timely per:
# - https://access.redhat.com/solutions/2045583
# - https://access.redhat.com/support/policy/updates/ceph-storage
community_to_release_map = {
    "0.80.8-5": {'version': "1.2.3", 'major': '1.2', 'minor': '3', 'downstream_release': 'NA'},
    "0.94.1-15": {'version': "1.3", 'major': '1.3', 'minor': '0', 'downstream_release': 'NA'},
    "0.94.3-3": {'version': "1.3.1", 'major': '1.3', 'minor': '1', 'downstream_release': 'NA'},
    "0.94.5-9": {'version': "1.3.2", 'major': '1.3', 'minor': '2', 'downstream_release': 'NA'},
    "0.94.5-12": {'version': "1.3.2", 'major': '1.3', 'minor': '2', 'downstream_release': 'async'},
    "0.94.5-13": {'version': "1.3.2", 'major': '1.3', 'minor': '2', 'downstream_release': 'async'},
    "0.94.5-14": {'version': "1.3.2", 'major': '1.3', 'minor': '2', 'downstream_release': 'async'},
    "0.94.5-15": {'version': "1.3.2", 'major': '1.3', 'minor': '2', 'downstream_release': 'async'},
    "0.94.9-3": {'version': "1.3.3", 'major': '1.3', 'minor': '3', 'downstream_release': 'NA'},
    "0.94.9-8": {'version': "1.3.3", 'major': '1.3', 'minor': '3', 'downstream_release': 'async'},
    "0.94.9-9": {'version': "1.3.3", 'major': '1.3', 'minor': '3', 'downstream_release': 'async 2'},
    "0.94.10-2": {'version': "1.3.4", 'major': '1.3', 'minor': '4', 'downstream_release': 'NA'},
    "10.2.2-38": {'version': "2.0", 'major': '2', 'minor': '0', 'downstream_release': '0'},
    "10.2.2-41": {'version': "2.0", 'major': '2', 'minor': '0', 'downstream_release': 'async'},
    "10.2.3-13": {'version': "2.1", 'major': '2', 'minor': '1', 'downstream_release': '0'},
    "10.2.3-17": {'version': "2.1", 'major': '2', 'minor': '1', 'downstream_release': 'async'},
    "10.2.5-37": {'version': "2.2", 'major': '2', 'minor': '2', 'downstream_release': '0'},
    "10.2.7-27": {'version': "2.3", 'major': '2', 'minor': '3', 'downstream_release': '0'},
    "10.2.7-28": {'version': "2.3", 'major': '2', 'minor': '3', 'downstream_release': 'async'},
    "10.2.7-32": {'version': "2.4", 'major': '2', 'minor': '4', 'downstream_release': '0'},
    "10.2.7-48": {'version': "2.4", 'major': '2', 'minor': '4', 'downstream_release': 'async'},
    "10.2.10-16": {'version': "2.5", 'major': '2', 'minor': '5', 'downstream_release': '0'},
    "10.2.10-17": {'version': "2.5", 'major': '2', 'minor': '5', 'downstream_release': 'async'},
    "10.2.10-28": {'version': "2.5.1", 'major': '2', 'minor': '5', 'downstream_release': '1'},
    "10.2.10-40": {'version': "2.5.2", 'major': '2', 'minor': '5', 'downstream_release': '2'},
    "10.2.10-43": {'version': "2.5.3", 'major': '2', 'minor': '5', 'downstream_release': '3'},
    "10.2.10-49": {'version': "2.5.4", 'major': '2', 'minor': '5', 'downstream_release': '4'},
    "10.2.10-51": {'version': "2.5.5", 'major': '2', 'minor': '5', 'downstream_release': '5', 'els': True},
    "12.2.1-40": {'version': "3.0", 'major': '3', 'minor': '0', 'downstream_release': '0'},
    "12.2.1-45": {'version': "3.0", 'major': '3', 'minor': '0', 'downstream_release': '1'},
    "12.2.1-46": {'version': "3.0", 'major': '3', 'minor': '0', 'downstream_release': '1 CVE'},
    "12.2.4-6": {'version': "3.0.2", 'major': '3', 'minor': '0', 'downstream_release': '2'},
    "12.2.4-10": {'version': "3.0.3", 'major': '3', 'minor': '0', 'downstream_release': '3'},
    "12.2.4-30": {'version': "3.0.4", 'major': '3', 'minor': '0', 'downstream_release': '4'},
    "12.2.4-42": {'version': "3.0.5", 'major': '3', 'minor': '0', 'downstream_release': '5'},
    "12.2.5-42": {'version': "3.1", 'major': '3', 'minor': '1', 'downstream_release': '0'},
    "12.2.5-59": {'version': "3.1.1", 'major': '3', 'minor': '1', 'downstream_release': '1'},
    "12.2.8-52": {'version': "3.2", 'major': '3', 'minor': '2', 'downstream_release': '0'},
    "12.2.8-89": {'version': "3.2.1", 'major': '3', 'minor': '2', 'downstream_release': '1'},
    "12.2.8-128": {'version': "3.2.2", 'major': '3', 'minor': '2', 'downstream_release': '2'},
    "12.2.12-45": {'version': "3.3", 'major': '3', 'minor': '3', 'downstream_release': '0'},
    "12.2.12-48": {'version': "3.3", 'major': '3', 'minor': '3', 'downstream_release': 'async'},
    "12.2.12-74": {'version': "3.3.1", 'major': '3', 'minor': '3', 'downstream_release': '1'},
    "12.2.12-79": {'version': "3.3.1", 'major': '3', 'minor': '3', 'downstream_release': 'async'},
    "12.2.12-84": {'version': "3.3.2", 'major': '3', 'minor': '3', 'downstream_release': '2'},
    "12.2.12-101": {'version': "3.3.4", 'major': '3', 'minor': '3', 'downstream_release': '4'},
    "12.2.12-115": {'version': "3.3.5", 'major': '3', 'minor': '3', 'downstream_release': '0'},
    "12.2.12-124": {'version': "3.3.6", 'major': '3', 'minor': '3', 'downstream_release': '0'},
    "12.2.12-127": {'version': "3.3.6", 'major': '3', 'minor': '3', 'downstream_release': '0'},
    "12.2.12-139": {'version': "3.3.7", 'major': '3', 'minor': '3', 'downstream_release': '0', 'els': True},
    "12.2.12-141": {'version': "3.3.8", 'major': '3', 'minor': '3', 'downstream_release': '0', 'els': True},
    "14.2.4-125": {'version': "4.0", 'major': '4', 'minor': '0', 'downstream_release': '0'},
    "14.2.4-51": {'version': "4.0", 'major': '4', 'minor': '0', 'downstream_release': '0'},
    "14.2.8-50": {'version': "4.1", 'major': '4', 'minor': '1', 'downstream_release': '0'},
    "14.2.8-59": {'version': "4.1", 'major': '4', 'minor': '1', 'downstream_release': '0'},
    "14.2.8-81": {'version': "4.1.1", 'major': '4', 'minor': '1', 'downstream_release': '0'},
    "14.2.8-89": {'version': "4.1.1", 'major': '4', 'minor': '1', 'downstream_release': 'async'},
    "14.2.8-91": {'version': "4.1.1", 'major': '4', 'minor': '1', 'downstream_release': 'async 2'},
    "14.2.8-111": {'version': "4.1.2", 'major': '4', 'minor': '1', 'downstream_release': '0'},
    "14.2.8-115": {'version': "4.1.3", 'major': '4', 'minor': '1', 'downstream_release': '0'},
    "14.2.11-95": {'version': "4.2", 'major': '4', 'minor': '2', 'downstream_release': '0'},
    "14.2.11-139": {'version': "4.2", 'major': '4', 'minor': '2', 'downstream_release': 'async'},
    "14.2.11-147": {'version': "4.2.1", 'major': '4', 'minor': '2', 'downstream_release': '0'},
    "14.2.11-181": {'version': "4.2.2", 'major': '4', 'minor': '2', 'downstream_release': '0'},
    "14.2.11-184": {'version': "4.2.2", 'major': '4', 'minor': '2', 'downstream_release': 'async'},
    "14.2.11-199": {'version': "4.2.3", 'major': '4', 'minor': '2', 'downstream_release': '0'},
    "14.2.11-208": {'version': "4.2.4", 'major': '4', 'minor': '2', 'downstream_release': '0'},
    "14.2.22-110": {'version': "4.3", 'major': '4', 'minor': '3', 'downstream_release': '0'},
    "14.2.22-128": {'version': "4.3.1", 'major': '4', 'minor': '3', 'downstream_release': '1'},
    "16.2.0-117": {'version': "5.0", 'major': '5', 'minor': '0', 'downstream_release': '0'},
    "16.2.0-143": {'version': "5.0.1", 'major': '5', 'minor': '0', 'downstream_release': '0'},
    "16.2.0-146": {'version': "5.0.3", 'major': '5', 'minor': '0', 'downstream_release': '0'},
    "16.2.0-152": {'version': "5.0.4", 'major': '5', 'minor': '0', 'downstream_release': '0'},
    "16.2.7-98": {'version': "5.1", 'major': '5', 'minor': '1', 'downstream_release': '0'},
    "16.2.7-112": {'version': "5.1.1", 'major': '5', 'minor': '1', 'downstream_release': '0'},
    "16.2.7-126": {'version': "5.1.2", 'major': '5', 'minor': '1', 'downstream_release': '2'},
    "16.2.8-84": {'version': "5.2", 'major': '5', 'minor': '2', 'downstream_release': '0'},
    "16.2.8-85": {'version': "5.2", 'major': '5', 'minor': '2', 'downstream_release': 'async'},
    "16.2.10-94": {'version': "5.3", 'major': '5', 'minor': '3', 'downstream_release': '0'},
    "16.2.10-138": {'version': "5.3.1", 'major': '5', 'minor': '3', 'downstream_release': '1'},
    "16.2.10-160": {'version': "5.3.2", 'major': '5', 'minor': '3', 'downstream_release': '2'},
    "16.2.10-172": {'version': "5.3.3", 'major': '5', 'minor': '3', 'downstream_release': '3'},
    "16.2.10-187": {'version': "5.3.4", 'major': '5', 'minor': '3', 'downstream_release': '4'},
    "16.2.10-208": {'version': "5.3.5", 'major': '5', 'minor': '3', 'downstream_release': '5'},
    "16.2.10-248": {'version': "5.3.6", 'major': '5', 'minor': '3', 'downstream_release': '6'},
    "16.2.10-266": {'version': "5.3.7", 'major': '5', 'minor': '3', 'downstream_release': '7'},
    "17.2.5-75": {'version': "6.0", 'major': '6', 'minor': '0', 'downstream_release': '0'},
    "17.2.6-70": {'version': "6.1", 'major': '6', 'minor': '1', 'downstream_release': '0'},
    "17.2.6-100": {'version': "6.1.1", 'major': '6', 'minor': '1', 'downstream_release': '1'},
    "17.2.6-148": {'version': "6.1.2", 'major': '6', 'minor': '1', 'downstream_release': '2'},
    "17.2.6-167": {'version': "6.1.3", 'major': '6', 'minor': '1', 'downstream_release': '3'},
    "17.2.6-170": {'version': "6.1.3", 'major': '6', 'minor': '1', 'downstream_release': 'async'},
    "17.2.6-196": {'version': "6.1.4", 'major': '6', 'minor': '1', 'downstream_release': '4'},
    "17.2.6-209": {'version': "6.1.5", 'major': '6', 'minor': '1', 'downstream_release': '5'},
    "17.2.6-216": {'version': "6.1.6", 'major': '6', 'minor': '1', 'downstream_release': '6'},
    "17.2.6-246": {'version': "6.1.7", 'major': '6', 'minor': '1', 'downstream_release': '7'},
    "18.2.0-131": {'version': "7.0", 'major': '7', 'minor': '0', 'downstream_release': '0'},
    "18.2.0-153": {'version': "7.0.1", 'major': '7', 'minor': '0', 'downstream_release': '1'},
    "18.2.0-192": {'version': "7.0.2", 'major': '7', 'minor': '0', 'downstream_release': '2'},
    "18.2.1-194": {'version': "7.1", 'major': '7', 'minor': '1', 'downstream_release': '0'},
    "18.2.1-229": {'version': "7.1.1", 'major': '7', 'minor': '1', 'downstream_release': '1'},
    "18.2.1-262": {'version': "7.1.2", 'major': '7', 'minor': '1', 'downstream_release': '2'},
    "19.2.0-53": {'version': "8.0", 'major': '8', 'minor': '0', 'downstream_release': '0'},
    "19.2.0-55": {'version': "8.0.1", 'major': '8', 'minor': '0', 'downstream_release': '1'},
    "19.2.0-98": {'version': "8.0.2", 'major': '8', 'minor': '0', 'downstream_release': '2'},
    "19.2.0-124": {'version': "8.0.3", 'major': '8', 'minor': '0', 'downstream_release': '3'},
    "19.2.0-137": {'version': "8.0.4", 'major': '8', 'minor': '0', 'downstream_release': '4'},
    "19.2.1-222": {'version': "8.1", 'major': '8', 'minor': '1', 'downstream_release': '0'},
}


@parser(Specs.ceph_v)
class CephVersion(CommandParser):
    """
    Class for parsing the output of command ``ceph -v``.

    Typical output of the ``ceph -v`` command is::

        ceph version 0.94.9-9.el7cp (b83334e01379f267fb2f9ce729d74a0a8fa1e92c)

    Attributes:
        version (str): The Red Hat release version
        major (str): The major version of Red Hat release version
        minor (str): The minor version of Red Hat release version
        is_els (boolean): If the verion in 'Extended life cycle support (ELS) add-on' phase
        downstream_release (str): The downstream release info
        upstream_version (dict): The detailed upstream version info with the
            following keys `release (int)`, `major (int)` and `minor (int)`.

    Example:
        >>> ceph_v.version
        '1.3.3'
        >>> ceph_v.major
        '1.3'
        >>> ceph_v.minor
        '3'
        >>> ceph_v.is_els
        False
    """

    def parse_content(self, content):
        # Parse Ceph Version Content and get Release, Major, Minor number
        if not content or len(content) != 1:
            raise SkipComponent("Empty Ceph Version Line", content)
        community_version = get_community_version(content[0])
        release_data = get_ceph_version(community_version)

        self.version = release_data['version']
        self.major = release_data['major']
        self.minor = release_data['minor']
        self.is_els = release_data.get('els', False)
        self.downstream_release = release_data['downstream_release']
        self.upstream_version = release_data['upstream_version']


def get_community_version(version_full):
    """
    Returns the community version part from the output of ``ceph -v``
    """
    if any(item not in version_full for item in ['ceph version', '.', '-']):
        raise SkipComponent("Wrong Format Ceph Version", version_full)

    return version_full.split()[2]


def get_ceph_version(community_full):
    """
    Get the release version from the mapping metrics and generate the
    `upstream_version`.
    """
    community_version, _ = rsplit(community_full, '.')
    release_data = community_to_release_map.get(community_version, None)
    if not release_data:
        raise SkipComponent("No Mapping Release Version", community_version)
    release_data['upstream_version'] = dict(
        zip(
            ['release', 'major', 'minor'],
            [int(_i) for _i in community_version.split('-')[0].split('.')])
    )
    return release_data



"""
lsinitrd - command ``lsinitrd``
===============================

This module contains the following parsers:

Lsinitrd - command ``lsinitrd``
-------------------------------

LsinitrdKdumpImage - command ``lsinitrd initramfs-<kernel-version>kdump.img``
-----------------------------------------------------------------------------

LsinitrdLvmConf - command ``/bin/lsinitrd -f /etc/lvm/lvm.conf --kver {default_kernel_version}``
------------------------------------------------------------------------------------------------

"""

from insights import parser, CommandParser
from insights.core import ls_parser
from insights.specs import Specs
from insights.parsers import keyword_search
from insights.parsers.lvm import LvmConf


@parser(Specs.lsinitrd)
class Lsinitrd(CommandParser):
    """
    This parser parses the filtered output of command ``lsinitrd`` and provides
    the info of listed files.

    A parser for command "lsinitrd".

    Attributes:
        data (dict): The key is the filename, the value is a dict describe
            the file's info.
        unparsed_lines(list): List of strings for unparsed lines.

    As this lsinitrd spec is set to filterable, the structure of the output
    is broken. Hence, this parser will parse only filelisting like lines in
    output of 'lisinitrd', and also store all the unparsed lines.
    If the other parts of the output structure are required in the future,
    an enhancement may be performed then.

    Examples:
        >>> len(ls.data)
        5
        >>> assert ls.search(name__contains='kernel') == [
        ...    {'group': 'root', 'name': 'kernel/x86', 'links': 3, 'perms': 'rwxr-xr-x',
        ...     'owner': 'root', 'date': 'Apr 20 15:58', 'type': 'd', 'dir': '', 'size': 0}
        ... ]
        >>> "udev-rules" in ls.unparsed_lines
        True
    """

    def parse_content(self, content):
        file_types = set(['s', 'd', 'p', 'l', '-', 'c', 'b'])
        perm = set(['-w-', 'rw-', '--x', '-wx', '---', 'rwx', 'r--', 'r-x'])
        entries = []
        _unparsed_lines = []

        for l in content:
            if l and len(l) > 10 and l[0] in file_types and l[1:4] in perm:
                entries.append(l)
            else:
                _unparsed_lines.append(l)
        self.unparsed_lines = _unparsed_lines

        d = ls_parser.parse(entries, '').get('')
        self.data = d.get('entries')

    def search(self, **kwargs):
        """
        Search the listed files for matching rows based on key-value pairs.

        This uses the :py:func:`insights.parsers.keyword_search` function for
        searching; see its documentation for usage details.  If no search
        parameters are given, no rows are returned.

        Returns:
            list: A list of dictionaries of files that match the given
            search criteria.

        Examples:
            >>> lsdev = ls.search(name__contains='dev')
            >>> len(lsdev)
            3
            >>> dev_console = {
            ...     'type': 'c', 'perms': 'rw-r--r--', 'links': 1, 'owner': 'root', 'group': 'root',
            ...     'major': 5, 'minor': 1, 'date': 'Apr 20 15:57', 'name': 'dev/console', 'dir': '',
            ... }
            >>> dev_console in lsdev
            True
            >>> 'dev/kmsg' in [l['name'] for l in lsdev]
            True
            >>> 'dev/null' in [l['name'] for l in lsdev]
            True

        """
        return keyword_search(self.data.values(), row_keys_change=True, **kwargs)


@parser(Specs.lsinitrd_kdump_image)
class LsinitrdKdumpImage(Lsinitrd):
    """
    Parses output of ``lsinitrd initramfs-<kernel-version>kdump.img`` command.

    Sample ``lsinitrd initramfs-<kernel-version>kdump.img`` output::

        Image: initramfs-4.18.0-240.el8.x86_64kdump.img: 19M
        ========================================================================
        Version: dracut-049-95.git20200804.el8

        Arguments: --quiet --hostonly --hostonly-cmdline --hostonly-i18n --hostonly-mode 'strict' -o 'plymouth dash resume ifcfg earlykdump' --mount '/dev/mapper/rhel-root /sysroot xfs rw,relatime,seclabel,attr2,inode64,logbufs=8,logbsize=32k,noquota,nofail,x-systemd.before=initrd-fs.target' --no-hostonly-default-device -f

        dracut modules:
        bash
        systemd
        systemd-initrd
        i18n
        ========================================================================
        crw-r--r--   1 root     root       5,   1 Aug  4  2020 dev/console
        crw-r--r--   1 root     root       1,  11 Aug  4  2020 dev/kmsg
        crw-r--r--   1 root     root       1,   3 Aug  4  2020 dev/null
        crw-r--r--   1 root     root       1,   8 Aug  4  2020 dev/random
        crw-r--r--   1 root     root       1,   9 Aug  4  2020 dev/urandom
        drwxr-xr-x  14 root     root            0 Aug  4  2020 .
        lrwxrwxrwx   1 root     root            7 Aug  4  2020 bin -> usr/bin
        drwxr-xr-x   2 root     root            0 Aug  4  2020 dev
        ========================================================================

    Examples:
        >>> type(lsinitrd_kdump_image)
        <class 'insights.parsers.lsinitrd_kdump_image.LsinitrdKdumpImage'>
        >>> lsinitrd_kdump_image.search(name__contains='urandom')
        >>> len(result_list)
        1
        >>> result_list[0].get('raw_entry')
        'crw-r--r--   1 root     root       1,   9 Aug  4  2020 dev/urandom'
    """

    pass


@parser(Specs.lsinitrd_lvm_conf)
class LsinitrdLvmConf(LvmConf):
    """
    Parses the ``/dev/lvm/lvm.conf`` file get from the default initramfs.

    Sample Input::
        # volume_list = [ "vg1", "vg2/lvol1", "@tag1", "@*" ]
        volume_list = [ "vg2", "vg3/lvol3", "@tag2", "@*" ]

    Examples:
        >>> type(lsinitrd_lvm_conf)
        <class 'insights.parsers.lsinitrd.LsinitrdLvmConf'>
        >>> lsinitrd_lvm_conf.get("volume_list")
        [ "vg2", "vg3/lvol3", "@tag2", "@*" ]
    """

    pass



# -*- coding: utf-8 -*-

from insights.client.phase import v1
from insights.tests.helpers import dummy_requests_session, getenv_bool, Timeout
from mock.mock import patch
from pytest import mark


RUN_TESTS = getenv_bool("TEST_NET_COMM", False)
MAX_TIME = 30  # The tests mustn't run longer even if all requests wait for timeout.


def load_all(self):
    return self


@mark.skipif(not RUN_TESTS, reason="Net communication tests take a long time to finish.")
@patch("insights.client.connection.requests.Session", dummy_requests_session)
@patch("insights.client.config.InsightsConfig.load_all", load_all)
def test_failure_does_not_take_too_long():
    """
    Client execution does not take too long if connection is blocked by a firewall. First thing that fails is the new
    egg fetch in the update phase. Every net request up to this point must timeout in reasonable time.
    """
    def worker():
        """
        Runs first two phases. Expected to be run in a subprocess. The update phase is expected to fail.
        """
        try:
            v1.pre_update()
        except SystemExit as exception:
            if exception.code:
                exit(1)  # This phase must not fail.
        else:
            exit(1)  # Phase did not exit.

        try:
            v1.update()
        except SystemExit as exception:
            if exception.code != 1:
                exit(1)  # Egg update is expected to fail.
        else:
            exit(1)  # Phase did not exit.

    timeout = Timeout()
    timeout.start(worker, MAX_TIME)

    assert timeout.state != timeout.FAIL
    assert timeout.state != timeout.TIMEOUT
    assert timeout.state == timeout.SUCCESS



from insights.core.exceptions import SkipComponent
from insights.parsers import xfs_quota
from insights.parsers.xfs_quota import XFSQuotaState
from insights.tests import context_wrap
import doctest
import pytest

XFSQUOTASTATE_ERR = """
""".strip()

XFSQUOTASTATE = """
User quota state on /sdd (/dev/sdd)
  Accounting: ON
  Enforcement: ON
  Inode: #131 (1 blocks, 1 extents)
Blocks grace time: [7 days]
Blocks max warnings: 5
Inodes grace time: [7 days]
Inodes max warnings: 5
Realtime Blocks grace time: [7 days]
Group quota state on /sdd (/dev/sdd)
  Accounting: OFF
  Enforcement: OFF
  Inode: N/A
Blocks grace time: [--------]
Blocks max warnings: 0
Inodes grace time: [--------]
Inodes max warnings: 0
Realtime Blocks grace time: [--------]
User quota state on /sdb (/dev/sdb)
  Accounting: ON
  Enforcement: ON
  Inode: #131 (2 blocks, 2 extents)
Blocks grace time: [6 days]
Blocks max warnings: 10
Inodes grace time: [6 days]
Inodes max warnings: 5
Realtime Blocks grace time: [6 days]
Group quota state on /sdb (/dev/sdb)
  Accounting: ON
  Enforcement: ON
  Inode: #137 (2 blocks, 2 extents)
Blocks grace time: [7 days]
Blocks max warnings: 5
Inodes grace time: [7 days]
Inodes max warnings: 5
Realtime Blocks grace time: [7 days]
User quota state on /sdc (/dev/sdc)
  Accounting: ON
  Enforcement: ON
  Inode: #131 (2 blocks, 2 extents)
Blocks grace time: [7 days]
Blocks max warnings: 20
Inodes grace time: [7 days]
Inodes max warnings: 5
Realtime Blocks grace time: [7 days]
Group quota state on /sdc (/dev/sdc)
  Accounting: ON
  Enforcement: ON
  Inode: #132 (2 blocks, 2 extents)
Blocks grace time: [7 days]
Blocks max warnings: 5
Inodes grace time: [7 days]
Inodes max warnings: 5
Realtime Blocks grace time: [7 days]
""".strip()


def test_repquota():
    results = XFSQuotaState(context_wrap(XFSQUOTASTATE))
    assert len(results.group_quota) == 3
    assert results.group_quota['/dev/sdd']['device'] == '/dev/sdd'
    assert results.group_quota['/dev/sdd']['accounting'] == 'OFF'
    assert results.group_quota['/dev/sdd']['blocks_grace_time'] is None
    assert len(results.user_quota) == 3
    assert results.group_quota['/dev/sdc']['accounting'] == 'ON'
    assert results.user_quota['/dev/sdc'] == {'device': '/dev/sdc', 'accounting': 'ON', 'enforcement': 'ON', 'inode': '#131 (2 blocks, 2 extents)', 'blocks_grace_time': '7 days', 'blocks_max_warnings': '20', 'inodes_grace_time': '7 days', 'inodes_max_warnings': '5', 'realtime_blocks_grace_time': '7 days'}


def test_repquota_err():
    with pytest.raises(SkipComponent):
        XFSQuotaState(context_wrap(XFSQUOTASTATE_ERR))


def test_repquota_doc_examples():
    env = {
        'quota_state': XFSQuotaState(context_wrap(XFSQUOTASTATE))
    }
    failed, total = doctest.testmod(xfs_quota, globs=env)
    assert failed == 0



import pytest

from collections import defaultdict

try:
    from unittest.mock import patch
except Exception:
    from mock import patch

from insights.core import filters
from insights.core.exceptions import SkipComponent
from insights.parsers.blkid import BlockIDInfo
from insights.parsers.fstab import FSTab
from insights.parsers.lvm import Pvs
from insights.specs import Specs
from insights.specs.datasources.ls import (
    list_files_with_lH,
    list_with_la,
    list_with_la_filtered,
    list_with_lan,
    list_with_lan_filtered,
    list_with_lanL,
    list_with_lanR,
    list_with_lanRL,
    list_with_laRZ,
    list_with_laZ,
)
from insights.tests import context_wrap


FSTAB_CONTEXT = """
/dev/mapper/rhel_rhel8-root /                       xfs     defaults        0 0
/dev/vda1 /boot                   xfs     defaults        0 0
/dev/mapper/rhel_rhel8-swap none                    swap    defaults        0 0
/dev/mapper/rhel_rhel7-hana1 /hana/data/rhel7-hana1                       xfs     defaults        0 0
/dev/mapper/rhel_rhel7-hana2 /hana/data/rhel7-hana2                       xfs     defaults        0 0
/dev/mapper/rhel_rhel7-hana3 /hana/data/rhel7-hana3                       xfs     defaults        0 0
""".strip()

FSTAB_DATA = """
/dev/sdb2                                 /var                       ext3    defaults        1 1
/dev/mapper/rhel-root                     /                          ext4    defaults        1 1
UUID=9c4f1e8e-60c8-46ef-b70b-111111111111 /home                      xfs     defaults        0 0
LABEL=new-label         /var                    ext3    defaults        1 2
/dev/archive_data           /data/ac/archive none    bind                    0 0
""".strip()

BLKID_DATA = """
/dev/sda1: LABEL="TestHome" TYPE="ext4"
/dev/sdb1: LABEL="" UUID="c7c45f2d-1d1b-4cf0-9d51-e2b0046682f8" TYPE="ext4"
/dev/sdb2: LABEL="" UUID="UVTk76-UWOc-vk7s-galL-dxIP-4UXO-0jG4MH" TYPE="ext4"
/dev/mapper/rhel-home: LABEL="" UUID="9c4f1e8e-60c8-46ef-b70b-111111111111" TYPE="ext4"
/dev/mapper/rhel-var: LABEL="new-label" UUID="9c4f1e8e-60c8-46ef-b70b-5a7ae1fa3b7f" TYPE="xfs"
""".strip()

PVS_DATA = """
  WARNING: locking_type (0) is deprecated, using --nolocking.
  WARNING: File locking is disabled.
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='1.00g'|LVM2_PV_NAME='/dev/vda1'|LVM2_PV_MAJOR='252'|LVM2_PV_MINOR='1'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_PV_DEVICE_ID=''|LVM2_PV_DEVICE_ID_TYPE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='D0bk3H-NASe-ncYz-L3T0-mPSH-QbzT-xlPTrh'|LVM2_DEV_SIZE='<127.00g'|LVM2_PV_NAME='/dev/vda2'|LVM2_PV_MAJOR='252'|LVM2_PV_MINOR='2'|LVM2_PV_MDA_FREE='507.50k'|LVM2_PV_MDA_SIZE='1020.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='1.00m'|LVM2_PV_SIZE='<127.00g'|LVM2_PV_FREE='<54.94g'|LVM2_PV_USED='<72.06g'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='32511'|LVM2_PV_PE_ALLOC_COUNT='18447'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_PV_DEVICE_ID=''|LVM2_PV_DEVICE_ID_TYPE=''|LVM2_VG_NAME='rhel_rhel8'
""".strip()


def setup_function(func):
    if func is test_la_empty:
        pass
    if func is test_la:
        filters.add_filter(Specs.ls_la_dirs, ["/"])
    if func is test_la_filterd:
        filters.add_filter(Specs.ls_la_filtered_dirs, ["/", '/boot'])
    if func is test_lan:
        filters.add_filter(Specs.ls_lan_dirs, ["/", '/boot'])
    if func is test_lan_filterd:
        filters.add_filter(Specs.ls_lan_filtered_dirs, ["/"])
    if func is test_lanL:
        filters.add_filter(Specs.ls_lanL_dirs, ["/", '/boot'])
    if func is test_lanR:
        filters.add_filter(Specs.ls_lanR_dirs, ["/"])
    if func is test_lanRL:
        filters.add_filter(Specs.ls_lanRL_dirs, ["/", '/boot'])
    if func is test_lanRZ:
        filters.add_filter(Specs.ls_laRZ_dirs, ['/boot'])
    if func is test_lanZ:
        filters.add_filter(Specs.ls_laZ_dirs, ["/", '/mnt'])
    if func is test_lan_with_fstab_mounted_filter:
        filters.add_filter(Specs.ls_lan_dirs, ["/", '/boot', 'fstab_mounted.dirs'])
    if func is test_lH_files:
        filters.add_filter(
            Specs.ls_lH_files,
            ["/etc/redhat-release", '/var/log/messages', 'fstab_mounted.devices', 'pvs.devices'],
        )
    if func is test_lH_files_pvs:
        filters.add_filter(
            Specs.ls_lH_files, ["/etc/redhat-release", '/var/log/messages', 'pvs.devices']
        )
    if func is test_lH_files_fstab_blkid:
        filters.add_filter(
            Specs.ls_lH_files, ["/etc/redhat-release", '/var/log/messages', 'fstab_mounted.devices']
        )


def teardown_function(func):
    filters._CACHE = {}
    filters.FILTERS = defaultdict(dict)


def test_la_empty():
    with pytest.raises(SkipComponent):
        list_with_la({})


def test_la():
    ret = list_with_la({})
    assert ret == '/ _non_existing_'


def test_la_filterd():
    ret = list_with_la_filtered({})
    assert ret == '/ /boot'


def test_lan():
    ret = list_with_lan({})
    assert ret == '/ /boot'


def test_lan_filterd():
    ret = list_with_lan_filtered({})
    assert ret == '/ _non_existing_'


def test_lanL():
    ret = list_with_lanL({})
    assert ret == '/ /boot'


def test_lanR():
    ret = list_with_lanR({})
    assert '/' == ret


def test_lanRL():
    ret = list_with_lanRL({})
    assert ret == '/ /boot'


def test_lanRZ():
    ret = list_with_laRZ({})
    assert ret == '/boot'


def test_lanZ():
    ret = list_with_laZ({})
    assert ret == '/ /mnt'


def test_lan_with_fstab_mounted_filter():
    ret = list_with_lan({})
    assert ret == '/ /boot fstab_mounted.dirs'

    fstab = FSTab(context_wrap(FSTAB_CONTEXT))
    broker = {FSTab: fstab}
    ret = list_with_lan(broker)
    assert ret == '/ /boot /hana/data'


@patch("os.path.isdir", return_value=False)
def test_lH_files(_):
    ret = list_files_with_lH({})
    assert ret == '/etc/redhat-release /var/log/messages fstab_mounted.devices pvs.devices'


@patch("os.path.isdir", return_value=False)
def test_lH_files_pvs(_):
    pvs_info = Pvs(context_wrap(PVS_DATA))
    broker = {Pvs: pvs_info}
    ret = list_files_with_lH(broker)
    assert ret == '/dev/vda1 /dev/vda2 /etc/redhat-release /var/log/messages'


@patch("os.path.isdir", return_value=False)
def test_lH_files_fstab_blkid(_):
    fstab_info = FSTab(context_wrap(FSTAB_DATA))
    blkid_info = BlockIDInfo(context_wrap(BLKID_DATA))
    broker = {FSTab: fstab_info, BlockIDInfo: blkid_info}
    ret = list_files_with_lH(broker)
    assert (
        ret
        == '/dev/mapper/rhel-home /dev/mapper/rhel-root /dev/mapper/rhel-var /dev/sdb2 /etc/redhat-release /var/log/messages'
    )



import doctest
from insights.parsers import sendmail
from insights.parsers.sendmail import SendmailMC
from insights.tests import context_wrap


SENDMAIL_MC_CONTENT = """
define(`confDEF_USER_ID', ``8:12'')dnl
dnl define(`confAUTO_REBUILD')dnl
define(`confTO_CONNECT', `1m')dnl
define(`confTRY_NULL_MX_LIST', `True')dnl
define(`confDONT_PROBE_INTERFACES', `True')dnl
define(`PROCMAIL_MAILER_PATH', `/usr/bin/procmail')dnl
define(`ALIAS_FILE', `/etc/aliases')dnl
define(`STATUS_FILE', `/var/log/mail/statistics')dnl
define(`UUCP_MAILER_MAX', `2000000')dnl
define(`confUSERDB_SPEC', `/etc/mail/userdb.db')dnl
define(`confPRIVACY_FLAGS', `authwarnings,novrfy,noexpn,restrictqrun')dnl
define(`confAUTH_OPTIONS', `A')dnl
"""

SendmailMC.last_scan('confTO_CONNECT_line', 'confTO_CONNECT')


def test_sendmail_mc():
    sendmail_mc_obj = SendmailMC(context_wrap(SENDMAIL_MC_CONTENT))

    assert sendmail_mc_obj.confTO_CONNECT_line
    assert sendmail_mc_obj.confTO_CONNECT_line.get('raw_line') == "define(`confTO_CONNECT', `1m')dnl"


def test_doc():
    env = {
            'sendmail_mc_obj': SendmailMC(context_wrap(SENDMAIL_MC_CONTENT))
          }
    failures, _ = doctest.testmod(sendmail, globs=env)
    assert failures == 0



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import localectl
from insights.parsers.localectl import LocaleCtlStatus
from insights.tests import context_wrap

LOCALECTL_STATUS_CONTENT = """
   System Locale: LANG=en_US.UTF-8
       VC Keymap: us
      X11 Layout: us
""".strip()

LOCALECTL_STATUS_CONTENT_ERROR = """
   command not fond
""".strip()

LOCALECTL_STATUS_CONTENT_EMPTY = """

""".strip()


def test_localectl_status():
    ret = LocaleCtlStatus(context_wrap(LOCALECTL_STATUS_CONTENT))
    assert ret['System Locale'] == 'LANG=en_US.UTF-8'
    assert ret['VC Keymap'] == 'us'
    assert ret['X11 Layout'] == 'us'


def test_localectl_status_error():
    with pytest.raises(SkipComponent):
        LocaleCtlStatus(context_wrap(LOCALECTL_STATUS_CONTENT_ERROR))


def test_localectl_status_ng():
    with pytest.raises(SkipComponent):
        LocaleCtlStatus(context_wrap(LOCALECTL_STATUS_CONTENT_EMPTY))


def test_doc_examples():
    env = {
            'localectl_status': LocaleCtlStatus(context_wrap(LOCALECTL_STATUS_CONTENT)),
          }
    failed, total = doctest.testmod(localectl, globs=env)
    assert failed == 0



"""
MAC Obfuscation
===============
"""

import hashlib
import logging
import os
import re
import six

from insights.cleaner.utilities import write_report

logger = logging.getLogger(__name__)


class Mac(object):
    """
    Class for obfuscating MAC addresses
    """

    def __init__(self):
        self._mac_db = dict()  # MAC database
        # Ignore list for MAC addresses
        # - 00:00:00:00:00:00
        # - FF:FF:FF:FF:FF:FF
        self._ignore_list = [r'\b(?:(?:00:){5}00|(?:ff:){5}ff)\b']
        # MAC address patterns
        self.pattern = r'(?<![0-9a-fA-F:-])([0-9a-fA-F]{2}([:-])(?:[0-9a-fA-F]{2}\2){4}[0-9a-fA-F]{2})(?![0-9a-fA-F:-])'

    def _mac2db(self, mac):
        '''
        Add an MAC address to MAC database and return obfuscated address.

        FORMAT:
        {$original_mac:, $obfuscated_mac}
        '''

        def obfuscate_hex(_hex, lower=True):
            __hex = _hex.encode('utf-8') if six.PY3 else _hex
            new_hex = hashlib.sha1(__hex.lower()).hexdigest()[: len(__hex)]
            return new_hex if lower else new_hex.upper()

        try:
            if mac in self._mac_db:
                return self._mac_db[mac]
            if mac in self._mac_db.values():  # pragma: no cover
                # avoid nested obfuscating
                return None
            lower = not mac.isupper()
            sep = '-' if '-' in mac else ':'
            self._mac_db[mac] = sep.join(obfuscate_hex(h, lower) for h in mac.split(sep))
            return self._mac_db[mac]
        except Exception as e:  # pragma: no cover
            logger.warning(e)
            raise Exception('SubMacError: Unable to Substitute MAC Addr - %s', mac)

    def parse_line(self, line, **kwargs):

        def _sub_mac(line, mac):
            new_mac = self._mac2db(mac)
            if new_mac:
                logger.debug("Obfuscating MAC Addr - %s > %s", mac, new_mac)
                return line.replace(mac, new_mac)
            # it's an obfuscated MAC address
            return line

        if not line:
            return line

        for mac in re.findall(self.pattern, line, re.I):
            if not any(re.search(_i, mac[0], re.I) for _i in self._ignore_list):
                line = _sub_mac(line, mac[0])

        return line

    def mapping(self):
        mapping = []
        for k, v in self._mac_db.items():
            mapping.append({'original': k, 'obfuscated': v})
        return mapping

    def generate_report(self, report_dir, archive_name):
        try:
            mac_report_file = os.path.join(report_dir, "%s-mac.csv" % archive_name)
            logger.info('Creating MAC addr Report - %s', mac_report_file)
            lines = ['Obfuscated MAC,Original MAC']
            for k, v in self._mac_db.items():
                lines.append('{0},{1}'.format(v, k))
        except Exception as e:  # pragma: no cover
            logger.exception(e)
            raise Exception('Error: Cannot creat MAC addr obfuscate report')

        write_report(lines, mac_report_file)

        logger.info('Completed MAC addr Report.')



import requests
import json
from insights.client.connection import InsightsConnection
from mock.mock import MagicMock, Mock, patch


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_registration_check_ok_reg(get_proxies, _init_session, _machine_id_exists, _generate_machine_id):
    '''
    Request completed OK, registered
        Returns True
    '''
    config = Mock(legacy_upload=False, base_url='example.com')
    conn = InsightsConnection(config)

    res = requests.Response()
    res._content = json.dumps(
        {
            'insights_id': 'xxxxxx',
            'id': 'yyyyyy'
        })
    res.status_code = 200

    conn.get = MagicMock(return_value=res)
    assert conn.api_registration_check()


@patch("insights.client.connection.machine_id_exists", return_value=False)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_registration_check_ok_unreg(get_proxies, _init_session, _machine_id_exists):
    '''
    Request completed OK, has not been registered
        Returns False
    '''
    config = Mock(legacy_upload=False, base_url='example.com')
    conn = InsightsConnection(config)

    res = requests.Response()
    res._content = json.dumps(
        {
            "count": 0,
            "page": 1,
            "per_page": 50,
            "results": [],
            "total": 0
        })
    res.status_code = 200

    conn.get = MagicMock(return_value=res)
    assert conn.api_registration_check() is False


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_registration_check_parse_error(get_proxies, _init_session, _machine_id_exists, _generate_machine_id):
    '''
    Can't parse response
        Returns None
    '''
    config = Mock(legacy_upload=False, base_url='example.com')
    conn = InsightsConnection(config)

    res = requests.Response()
    res._content = 'zSDFasfghsRGH'
    res.status_code = 200

    conn.get = MagicMock(return_value=res)
    assert conn.api_registration_check() is None


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_registration_check_no_host(get_proxies, _init_session, _machine_id_exists, _generate_machine_id):
    '''
    No hosts found in response
        Returns False
    '''
    config = Mock(legacy_upload=False, base_url='example.com')
    conn = InsightsConnection(config)

    res = requests.Response()
    res._content = json.dumps(
        {
            "title": "Not Found",
            "detail": "No host found for Insights ID 'xxxxxx'.",
            "insights_id": "xxxxxx",
        })
    res.status_code = 404

    conn.get = MagicMock(return_value=res)
    assert conn.api_registration_check() is False


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_registration_check_multiple_hosts(get_proxies, _init_session, _machine_id_exists, _generate_machine_id):
    '''
    Multiple hosts found in response
        Returns False
    '''
    config = Mock(legacy_upload=False, base_url='example.com')
    conn = InsightsConnection(config)

    res = requests.Response()
    res._content = json.dumps(
        {
            "title": "Conflict",
            "detail": "Multiple hosts with Insights ID 'xxxxxx' detected.",
            "insights_id": "xxxxxx",
        })
    res.status_code = 409

    conn.get = MagicMock(return_value=res)
    assert conn.api_registration_check() is False


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_registration_check_bad_res(get_proxies, _init_session, _machine_id_exists, _generate_machine_id):
    '''
    Failure HTTP response
        Returns None
    '''
    config = Mock(legacy_upload=False, base_url='example.com')
    conn = InsightsConnection(config)

    res = requests.Response()
    res._content = 'wakannai'
    res.status_code = 500

    conn.get = MagicMock(return_value=res)
    assert conn.api_registration_check() is None


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_registration_check_conn_error(get_proxies, _init_session, _machine_id_exists, _generate_machine_id):
    '''
    Connection error
        Returns None
    '''
    config = Mock(legacy_upload=False, base_url='example.com')
    conn = InsightsConnection(config)
    conn.get = MagicMock()
    conn.get.side_effect = requests.ConnectionError()
    assert conn.api_registration_check() is None



from insights.tests import context_wrap
from insights.parsers.lvdisplay import LvDisplay

LV_DISPLAY = """
      Adding lvsapp01ap01:0 as an user of lvsapp01ap01_mlog
  --- Volume group ---
  VG Name               vgp01app
  Format                lvm2
  Metadata Areas        4
  Metadata Sequence No  56
  VG Access             read/write
  VG Status             resizable
  Clustered             yes
  Shared                no
  MAX LV                0
  Cur LV                4
  Open LV               1
  Max PV                0
  Cur PV                4
  Act PV                4
  VG Size               399.98 GiB
  PE Size               4.00 MiB
  Total PE              102396
  Alloc PE / Size       82435 / 322.01 GiB
  Free  PE / Size       19961 / 77.97 GiB
  VG UUID               JVgCxE-UY84-C0Gk-8Cmn-UGXu-UHo0-9Qa4Re

  --- Logical volume ---
      global/lvdisplay_shows_full_device_path not found in config: defaulting to 0
  LV Path                /dev/vgp01app/lvsapp01ap01-old
  LV Name                lvsapp01ap01-old
  VG Name                vgp01app
  LV UUID                eLjsoG-Gvnh-zEbV-zFwD-HyQT-1zEs-VN4W2D
  LV Write Access        read/write
  LV Creation host, time lvn-itm-099, 2015-02-24 09:19:54 +0100
  LV Status              available
  # open                 0
  LV Size                64.00 GiB
  Current LE             16384
  Segments               4
  Allocation             inherit
  Read ahead sectors     auto
  - currently set to     256
  Block device           253:50

  --- Logical volume ---
      global/lvdisplay_shows_full_device_path not found in config: defaulting to 0
  LV Path                /dev/vgp01app/lvsapp01ap02
  LV Name                lvsapp01ap02
  VG Name                vgp01app
  LV UUID                tFGsSW-nimQ-4JUL-4Fw0-IJn0-Jcoo-Szgfgz
  LV Write Access        read/write
  LV Creation host, time lvn-itm-099, 2015-02-24 15:24:52 +0100
  LV Status              available
  # open                 0
  LV Size                64.00 GiB
  Current LE             16384
  Mirrored volumes       2
  Segments               1
  Allocation             inherit
  Read ahead sectors     auto
  - currently set to     256
  Block device           253:54
"""


def test_lvdisplay():
    lvs = LvDisplay(context_wrap(LV_DISPLAY))
    assert 'vgp01app' == lvs.get('volumes')['Volume group'][0]['VG Name']
    assert '399.98 GiB' == lvs.get('volumes')['Volume group'][0]['VG Size']
    assert 'vgp01app' == lvs.get('volumes')['Logical volume'][1]['VG Name']
    assert 'vgp01app' in lvs.vgs
    assert lvs.vgs['vgp01app'] == lvs.get('volumes')['Volume group'][0]
    assert 'lvsapp01ap02' in lvs.lvs
    assert lvs.lvs['lvsapp01ap02'] == lvs.get('volumes')['Logical volume'][1]



import doctest

from insights.parsers import selinux_config
from insights.parsers.selinux_config import SelinuxConfig
from insights.tests import context_wrap

SELINUX_CONFIG = """
# This file controls the state of SELinux on the system.
# SELINUX= can take one of these three values:
#     enforcing - SELinux security policy is enforced.
#     permissive - SELinux prints warnings instead of enforcing.
#     disabled - No SELinux policy is loaded.
SELINUX=enforcing

 # SELINUXTYPE= can take one of three two values:
 #     targeted - Targeted processes are protected,
 #     minimum - Modification of targeted policy. Only selected processes are protected.
 #     mls - Multi Level Security protection.
SELINUXTYPE=targeted

"""


def test_selinux_config():
    selinux_conf = SelinuxConfig(context_wrap(SELINUX_CONFIG))
    assert selinux_conf["SELINUX"] == 'enforcing'
    assert selinux_conf.get("SELINUXTYPE") == 'targeted'
    assert len(selinux_conf) == 2


def test_doc_examples():
    env = {
            'conf': SelinuxConfig(context_wrap(SELINUX_CONFIG))
          }
    failed, total = doctest.testmod(selinux_config, globs=env)
    assert failed == 0



import os

from insights.client import InsightsClient
from insights.client.constants import InsightsConstants
from mock.mock import call
from mock.mock import patch


@patch("insights.client.atexit.register")
@patch("insights.client.write_to_disk")
@patch("insights.client.get_parent_process")
def test_write_pidfile(get_parent_process, write_to_disk, register):
    '''
    Test writing of the pidfile when InsightsClient
    is called initially (when from_phase=False)
    '''
    InsightsClient(from_phase=False)
    write_to_disk.assert_has_calls((
        call(InsightsConstants.pidfile, content=str(os.getpid())),
        call(InsightsConstants.ppidfile, content=get_parent_process.return_value)
    ))


@patch("insights.client.atexit.register")
@patch("insights.client.write_to_disk")
def test_atexit_delete_pidfile(write_to_disk, register):
    '''
    Test delete of the pidfile is registered when InsightsClient
    is called initially (when from_phase=False)
    '''
    InsightsClient(from_phase=False)
    register.assert_has_calls((
        call(write_to_disk, InsightsConstants.pidfile, delete=True),
        call(write_to_disk, InsightsConstants.ppidfile, delete=True)
    ))


@patch("insights.client.write_to_disk")
@patch("insights.client.get_parent_process")
def test_write_pidfile_not_called(get_parent_process, write_to_disk):
    '''
    Test that the pidfile is not written when
    called from a phase (from-phase=True)
    '''
    InsightsClient(from_phase=True)
    get_parent_process.assert_not_called()
    write_to_disk.assert_not_called()


@patch("insights.client.atexit.register")
def test_atexit_delete_pidfile_not_called(register):
    '''
    Test that delete of the pidfile is not registered when
    called from a phase (from_phase=True)
    '''
    InsightsClient(from_phase=True)
    register.assert_not_called()



import json
import os
import yaml

from collections import defaultdict

import insights

from insights.core import filters
from insights.specs import Specs
from insights.tools import apply_spec_filters


JSON_content = """
{
    "commands": [
        {
            "command": "/bin/ps alxwww",
            "pattern": [],
            "symbolic_name": "ps_alxwww"
        },
        {
            "command": "/bin/ps aux",
            "pattern": [],
            "symbolic_name": "ps_aux"
        },
        {
            "command": "/bin/ps auxcww",
            "pattern": [],
            "symbolic_name": "ps_auxcww"
        }
    ],
    "files": [
        {
            "file": "/etc/redhat-release",
            "pattern": [],
            "symbolic_name": "redhat_release"
        },
        {
            "file": "/etc/yum.conf",
            "pattern": [],
            "symbolic_name": "yum_conf"
        },
        {
            "file": "/var/log/yum.log",
            "pattern": [],
            "symbolic_name": "yum_log"
        }
    ],
    "globs": [
        {
            "glob": "/etc/nginx/*.conf",
            "pattern": [],
            "symbolic_name": "nginx_conf"
        }
    ],
    "version": "2024-11-21T03:08:24.937321"
}
""".strip()
JSON_file = '/tmp/_test_just_test_uploader.json'
YAML_file = '/tmp/_test_just_test_filters_yaml.yaml'
yaml_file = os.path.join(os.path.dirname(insights.__file__), filters._filename)


def setup_function():
    filters.add_filter(Specs.ps_alxwww, ['COMMAND', 'CMD'])
    filters.add_filter(Specs.ps_aux, 'COMMAND')
    filters.add_filter(Specs.yum_conf, '[')
    filters.add_filter(Specs.yum_log, ['Installed', 'Updated', 'Erased'])
    with open(JSON_file, 'w') as f:
        f.write(JSON_content)


def teardown_function():
    if os.path.exists(JSON_file):
        os.remove(JSON_file)
    if os.path.exists(YAML_file):
        os.remove(YAML_file)
    if os.path.exists(yaml_file):
        os.remove(yaml_file)

    filters._CACHE = {}
    filters.FILTERS = defaultdict(dict)


def test_apply_specs_filters_json():
    apply_spec_filters.apply_filters("json", 'insights.parsers', JSON_file)

    count = 0
    with open(JSON_file, 'r') as f:
        ret = json.load(f)
        # ps_alxwww
        assert len(ret['commands'][0]['pattern']) == 2
        count += 1
        # ps_aux
        assert len(ret['commands'][1]['pattern']) == 1
        count += 1
        # ps_auxcww
        assert len(ret['commands'][2]['pattern']) == 0
        count += 1
        # redhat_release
        assert len(ret['files'][0]['pattern']) == 0
        count += 1
        # yum_conf
        assert len(ret['files'][1]['pattern']) == 1
        count += 1
        # yum_log
        assert len(ret['files'][2]['pattern']) == 3
        count += 1

    assert count == 6


def test_apply_specs_filters_yaml():
    apply_spec_filters.apply_filters("yaml", 'insights.parsers', YAML_file)

    count = 0
    with open(YAML_file, 'r') as f:
        ret = yaml.safe_load(f)
        # ps_alxwww
        assert len(ret['insights.specs.Specs.ps_alxwww']) == 2
        count += 1
        # ps_aux
        assert len(ret['insights.specs.Specs.ps_aux']) == 1
        count += 1
        # ps_auxcww
        assert 'insights.specs.Specs.ps_auxcww' not in ret
        count += 1
        # redhat_release
        assert 'insights.specs.Specs.redhat_release' not in ret
        count += 1
        # yum_conf
        assert len(ret['insights.specs.Specs.yum_conf']) == 1
        count += 1
        # yum_log
        assert len(ret['insights.specs.Specs.yum_log']) == 3
        count += 1

    assert count == 6

    apply_spec_filters.apply_filters("yaml", 'insights.parsers')

    count = 0
    with open(yaml_file, 'r') as f:
        ret = yaml.safe_load(f)
        # ps_alxwww
        assert len(ret['insights.specs.Specs.ps_alxwww']) == 2
        count += 1
        # ps_aux
        assert len(ret['insights.specs.Specs.ps_aux']) == 1
        count += 1
        # ps_auxcww
        assert 'insights.specs.Specs.ps_auxcww' not in ret
        count += 1
        # redhat_release
        assert 'insights.specs.Specs.redhat_release' not in ret
        count += 1
        # yum_conf
        assert len(ret['insights.specs.Specs.yum_conf']) == 1
        count += 1
        # yum_log
        assert len(ret['insights.specs.Specs.yum_log']) == 3
        count += 1

    assert count == 6


def test_apply_specs_filters_ab():
    ret = apply_spec_filters.apply_filters("test", 'insights.parsers', YAML_file)
    assert ret == 1

    ret = apply_spec_filters.apply_filters("yaml", '', YAML_file)
    assert ret == 1

    ret = apply_spec_filters.apply_filters("json", 'insights.parsers')
    assert ret == 1

    ret = apply_spec_filters.apply_filters("json", 'insights.parsers', './abc_test')
    assert ret == 1



"""
Custom datasources related to ``corosync``
"""

import os

from insights.combiners.redhat_release import RedHatRelease
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource


@datasource(HostContext, RedHatRelease)
def corosync_cmapctl_cmds(broker):
    """
    corosync-cmapctl use different arguments on RHEL7 and RHEL8.

    Returns:
        list: A list of related corosync-cmapctl commands based the RHEL version.
    """
    corosync_cmd = '/usr/sbin/corosync-cmapctl'
    if os.path.exists(corosync_cmd):
        rhel = broker.get(RedHatRelease)
        if rhel.major <= 6:
            # RHEL 6
            raise SkipComponent()
        elif rhel.major == 7:
            # RHEL 7
            return [
                corosync_cmd,
                ' '.join([corosync_cmd, '-d runtime.schedmiss.timestamp']),
                ' '.join([corosync_cmd, '-d runtime.schedmiss.delay']),
            ]
        else:
            # Others
            return [
                corosync_cmd,
                ' '.join([corosync_cmd, '-m stats']),
                ' '.join([corosync_cmd, '-C schedmiss']),
            ]
    raise SkipComponent()



import doctest

from insights.parsers import ls_dev
from insights.parsers.ls_dev import LsDev
from insights.tests import context_wrap

LS_DEV = """
/dev:
total 3
brw-rw----.  1 0  6 253,   0 Aug  4 16:56 dm-0
brw-rw----.  1 0  6 253,   1 Aug  4 16:56 dm-1
brw-rw----.  1 0  6 253,  10 Aug  4 16:56 dm-10
crw-rw-rw-.  1 0  5   5,   2 Aug  5  2016 ptmx
drwxr-xr-x.  2 0  0        0 Aug  4 16:56 pts
lrwxrwxrwx.  1 0  0       25 Oct 25 14:48 initctl -> /run/systemd/initctl/fifo

/dev/rhel:
total 0
drwxr-xr-x.  2 0 0  100 Jul 25 10:00 .
drwxr-xr-x. 23 0 0 3720 Jul 25 12:43 ..
lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 home -> ../dm-2
lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 root -> ../dm-0
lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 swap -> ../dm-1

/dev/mapper:
total 0
drwxr-xr-x.  2 0 0     140 Jul 25 10:00 .
drwxr-xr-x. 23 0 0    3720 Jul 25 12:43 ..
crw-------.  1 0 0 10, 236 Jul 25 10:00 control
lrwxrwxrwx.  1 0 0       7 Jul 25 10:00 docker-253:0-1443032-pool -> ../dm-3
lrwxrwxrwx.  1 0 0       7 Jul 25 10:00 rhel-home -> ../dm-2
lrwxrwxrwx.  1 0 0       7 Jul 25 10:00 rhel-root -> ../dm-0
lrwxrwxrwx.  1 0 0       7 Jul 25 10:00 rhel-swap -> ../dm-1
"""

LS_ALZR_DEV = """
/dev:
total 0
drwxr-xr-x. 19 root     root    system_u:object_r:device_t:s0                  3300 Sep 30 16:58 .
dr-xr-xr-x. 18 root     root    system_u:object_r:root_t:s0                     235 Apr 21  2023 ..
crw-r--r--.  1 root     root    system_u:object_r:autofs_device_t:s0        10, 235 Sep 30 16:58 autofs
drwxr-xr-x.  2 root     root    system_u:object_r:device_t:s0                   180 Sep 30 16:58 block
lrwxrwxrwx.  1 root     root    system_u:object_r:device_t:s0                    12 Sep 30 16:58 initctl -> /run/initctl
brw-rw----.  1 root     disk    system_u:object_r:fixed_disk_device_t:s0   252,   0 Sep 30 16:58 vda
brw-rw----.  1 root     disk    system_u:object_r:fixed_disk_device_t:s0   252,   1 Sep 30 16:58 vda1
brw-rw----.  1 root     disk    system_u:object_r:fixed_disk_device_t:s0   252,   2 Sep 30 16:58 vda2

/dev/mapper:
total 0
drwxr-xr-x.  2 root root system_u:object_r:device_t:s0          100 Sep 30 16:58 .
drwxr-xr-x. 19 root root system_u:object_r:device_t:s0         3300 Sep 30 16:58 ..
crw-------.  1 root root system_u:object_r:lvm_control_t:s0 10, 236 Sep 30 16:58 control
lrwxrwxrwx.  1 root root system_u:object_r:device_t:s0            7 Sep 30 16:58 rhel-root -> ../dm-0
lrwxrwxrwx.  1 root root system_u:object_r:device_t:s0            7 Sep 30 16:58 rhel-swap -> ../dm-1

/dev/rhel:
total 0
drwxr-xr-x.  2 root root system_u:object_r:device_t:s0   80 Sep 30 16:58 .
drwxr-xr-x. 19 root root system_u:object_r:device_t:s0 3300 Sep 30 16:58 ..
lrwxrwxrwx.  1 root root system_u:object_r:device_t:s0    7 Sep 30 16:58 root -> ../dm-0
lrwxrwxrwx.  1 root root system_u:object_r:device_t:s0    7 Sep 30 16:58 swap -> ../dm-1
"""


def test_ls_dev():
    lsdev = LsDev(context_wrap(LS_DEV))
    assert lsdev.listing_of("/dev/rhel") == {
        "home": {
            "group": "0",
            "name": "home",
            "links": 1,
            "perms": "rwxrwxrwx.",
            "owner": "0",
            "link": "../dm-2",
            "date": "Jul 25 10:00",
            "type": "l",
            "size": 7,
            "dir": "/dev/rhel",
        },
        "root": {
            "group": "0",
            "name": "root",
            "links": 1,
            "perms": "rwxrwxrwx.",
            "owner": "0",
            "link": "../dm-0",
            "date": "Jul 25 10:00",
            "type": "l",
            "size": 7,
            "dir": "/dev/rhel",
        },
        "swap": {
            "group": "0",
            "name": "swap",
            "links": 1,
            "perms": "rwxrwxrwx.",
            "owner": "0",
            "link": "../dm-1",
            "date": "Jul 25 10:00",
            "type": "l",
            "size": 7,
            "dir": "/dev/rhel",
        },
        "..": {
            "group": "0",
            "name": "..",
            "links": 23,
            "perms": "rwxr-xr-x.",
            "owner": "0",
            "date": "Jul 25 12:43",
            "type": "d",
            "size": 3720,
            "dir": "/dev/rhel",
        },
        ".": {
            "group": "0",
            "name": ".",
            "links": 2,
            "perms": "rwxr-xr-x.",
            "owner": "0",
            "date": "Jul 25 10:00",
            "type": "d",
            "size": 100,
            "dir": "/dev/rhel",
        },
    }
    expected = ["docker-253:0-1443032-pool", "rhel-home", "rhel-root", "rhel-swap"]
    actual = lsdev.files_of("/dev/mapper")
    assert actual == expected

    assert lsdev.get("/dev/mapper")["entries"]["rhel-home"]["link"] == "../dm-2"

    ls_alzr_dev = LsDev(context_wrap(LS_ALZR_DEV))
    assert "/dev/rhel" in ls_alzr_dev
    assert ls_alzr_dev.files_of("/dev") == ["initctl"]
    assert ls_alzr_dev.get("/dev")["specials"] == ["autofs", "vda", "vda1", "vda2"]
    assert ls_alzr_dev.get("/dev/rhel")["entries"].get("root") == {
        "type": "l",
        "perms": "rwxrwxrwx.",
        "links": 1,
        "owner": "root",
        "group": "root",
        "size": 7,
        "se_user": "system_u",
        "se_role": "object_r",
        "se_type": "device_t",
        "se_mls": "s0",
        "name": "root",
        "date": "Sep 30 16:58",
        "link": "../dm-0",
        "dir": "/dev/rhel",
    }
    assert (
        ls_alzr_dev.raw_entry_of("/dev/rhel", "root")
        == "lrwxrwxrwx. 1 root root system_u:object_r:device_t:s0 7 Sep 30 16:58 root -> ../dm-0"
    )


def test_doc_examples():
    env = {"ls_dev": LsDev(context_wrap(LS_DEV))}
    failed, total = doctest.testmod(ls_dev, globs=env)
    assert failed == 0



import json
from insights.client import InsightsClient
from insights.client.config import InsightsConfig
from insights.client.connection import InsightsConnection
from mock.mock import patch, Mock
import pytest

TEST_REMEDIATION_ID = 123456


class MockSession(object):
    def __init__(self):
        self.status_code = None
        self.text = None
        self.content = '{"big_dumb_error": "you_done_goofed"}'

    def get(self, url=None, timeout=None, headers=None, data=None, params=None):
        if params and params['remediation'] == TEST_REMEDIATION_ID:
            self.content = '{"specific_dumb_error": "stop_goofin"}'
        return MockResponse(self.status_code, self.text, self.content)

    def put(self, url=None, timeout=None, headers=None, data=None):
        return MockResponse(self.status_code, self.text, None)


class MockResponse(object):
    def __init__(self, status_code, text, content):
        self.status_code = status_code
        self.text = text
        self.content = content

    def json(self):
        return json.loads(self.content)


def mock_init_session(obj):
    return MockSession()


def mock_get_proxies(obj):
    return


@pytest.mark.skip()
@patch('insights.client.connection.InsightsConnection._init_session',
       mock_init_session)
@patch('insights.client.connection.InsightsConnection.get_proxies',
       mock_get_proxies)
@patch('insights.client.utilities.constants.machine_id_file',
       '/tmp/machine-id')
def test_get_diagnosis():
    conf = InsightsConfig()
    c = InsightsConnection(conf)
    c.session.status_code = 200
    assert c.get_diagnosis() == {'big_dumb_error': 'you_done_goofed'}
    c.session.status_code = 404
    assert c.get_diagnosis() is None
    c.session.status_code = 500
    c.session.text = 'oops'
    assert c.get_diagnosis() is None


@pytest.mark.skip()
@patch('insights.client.connection.InsightsConnection._init_session',
       mock_init_session)
@patch('insights.client.connection.InsightsConnection.get_proxies',
       mock_get_proxies)
@patch('insights.client.utilities.constants.machine_id_file',
       '/tmp/machine-id')
def test_get_diagnosis_with_id():
    conf = InsightsConfig()
    c = InsightsConnection(conf)
    c.session.status_code = 200
    assert c.get_diagnosis(TEST_REMEDIATION_ID) == {'specific_dumb_error': 'stop_goofin'}
    c.session.status_code = 404
    assert c.get_diagnosis() is None
    c.session.status_code = 500
    c.session.text = 'oops'
    assert c.get_diagnosis() is None


def test_get_diagnosis_offline():
    conf = InsightsConfig()
    conf.offline = True
    c = InsightsClient(conf)
    assert c.get_diagnosis() is None


@patch('insights.client.connection.InsightsConnection._init_session', Mock())
@patch('insights.client.connection.InsightsConnection.get_proxies', Mock())
@patch('insights.client.utilities.constants.machine_id_file', '/tmp/machine-id')
@patch("insights.client.connection.InsightsConnection._fetch_system_by_machine_id", return_value=True)
@patch('insights.client.connection.InsightsConnection.get')
def test_get_diagnosis_success(get, fetch_system_by_machine_id):
    '''
    Verify that fetching a diagnosis without an ID succeeds and
    returns a dict when HTTP response is valid
    '''
    conf = InsightsConfig()
    c = InsightsConnection(conf)
    get.return_value = MockResponse(status_code=200, text="OK", content="{\"test\": \"test\"}")
    assert c.get_diagnosis() == {"test": "test"}


@patch('insights.client.connection.InsightsConnection._init_session', Mock())
@patch('insights.client.connection.InsightsConnection.get_proxies', Mock())
@patch("insights.client.connection.InsightsConnection._fetch_system_by_machine_id", return_value=False)
def test_get_diagnosis_not_registered(fetch_system_by_machine_id):
    '''
    Verify that fetching a diagnosis fails when the system is not registered
    '''
    conf = InsightsConfig()
    c = InsightsConnection(conf)
    assert c.get_diagnosis() is False



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import db2
from insights.parsers.db2 import Db2ls
from insights.tests import context_wrap


DB2LS_A_C = """
#PATH:VRMF:FIXPACK:SPECIAL:INSTALLTIME:INSTALLERUID
/opt/ibm/db2/V11.5:11.5.6.0:0 ::Fri Jan 14 20:20:07 2022 CST :0
/opt/ibm/db2/V11.5_01:11.5.7.0:0 ::Fri Feb 11 10:34:51 2022 CST :0
"""

DB2LS_A_C_EMPTY1 = ""
DB2LS_A_C_EMPTY2 = "#PATH:VRMF:FIXPACK:SPECIAL:INSTALLTIME:INSTALLERUID"


DB2_CONFIGURATION_USER = """

       Database Configuration for Database TESTD1

 Database configuration release level                    = 0x1500
 Database release level                                  = 0x1500

 Update to database level pending                        = NO (0x0)
 Database territory                                      = US
 Database code page                                      = 1208
 Database code set                                       = UTF-8
 Database country/region code                            = 1
 Database collating sequence                             = IDENTITY
 Alternate collating sequence              (ALT_COLLATE) =
 Database page size                                      = 4096

 Statement concentrator                      (STMT_CONC) = OFF

 Discovery support for this database       (DISCOVER_DB) = ENABLE

 Restrict access                                         = NO
 Default query optimization class         (DFT_QUERYOPT) = 5
 Degree of parallelism                      (DFT_DEGREE) = 1
 Continue upon arithmetic exceptions   (DFT_SQLMATHWARN) = NO
 Default refresh age                   (DFT_REFRESH_AGE) = 0
 Default maintained table types for opt (DFT_MTTB_TYPES) = SYSTEM
 Number of frequent values retained     (NUM_FREQVALUES) = 10
 Number of quantiles retained            (NUM_QUANTILES) = 20

 Decimal floating point rounding mode  (DECFLT_ROUNDING) = ROUND_HALF_EVEN

 DECIMAL arithmetic mode                (DEC_ARITHMETIC) =
 Large aggregation                   (LARGE_AGGREGATION) = NO

 Backup pending                                          = NO

 All committed transactions have been written to disk    = YES
 Rollforward pending                                     = NO
 Restore pending                                         = NO
 Control file recovery path       (CTRL_FILE_RECOV_PATH) =
 Encrypted database                                      = NO
 Procedural language stack trace        (PL_STACK_TRACE) = NONE
 HADR SSL certificate label             (HADR_SSL_LABEL) =
 HADR SSL Hostname Validation        (HADR_SSL_HOST_VAL) = OFF

 BUFFPAGE size to be used by optimizer   (OPT_BUFFPAGE) = 0
 LOCKLIST size to be used by optimizer   (OPT_LOCKLIST) = 0
 MAXLOCKS size to be used by optimizer   (OPT_MAXLOCKS) = 0
 SORTHEAP size to be used by optimizer   (OPT_SORTHEAP) = 0
 SORTHEAP size to be used by optimizer   (opt_SORTHEAP) = 0
""".strip()

DB2_CONFIGURATION_USER_EMPTY = ""

DB2_CONFIGURATION_USER_INCORRECT = """

       Database Configuration for Database TESTD1

""".strip()


DB2_MANAGER_CONFIGURATION = """

          Database Manager Configuration

     Node type = Enterprise Server Edition with local and remote clients

 Database manager configuration release level            = 0x1500

 CPU speed (millisec/instruction)             (CPUSPEED) = 3.306409e-07
 Communications bandwidth (MB/sec)      (COMM_BANDWIDTH) = 1.000000e+02

 Max number of concurrently active databases     (NUMDB) = 32
 Federated Database System Support           (FEDERATED) = NO
 Transaction processor monitor name        (TP_MON_NAME) =

 Default charge-back account           (DFT_ACCOUNT_STR) =

 Java Development Kit installation path       (JDK_PATH) = /home/dbp/sqllib/java/jdk64

 Diagnostic error capture level              (DIAGLEVEL) = 3
 Notify Level                              (NOTIFYLEVEL) = 3
 Diagnostic data directory path               (DIAGPATH) = /home/dbp/sqllib/db2dump/ $m
 Current member resolved DIAGPATH                        = /home/dbp/sqllib/db2dump/DIAG0000/
 Alternate diagnostic data directory path (ALT_DIAGPATH) =
 Current member resolved ALT_DIAGPATH                    =
 Size of rotating db2diag & notify logs (MB)  (DIAGSIZE) = 0

 Default database monitor switches
   Buffer pool                         (DFT_MON_BUFPOOL) = OFF
   Lock                                   (DFT_MON_LOCK) = OFF
   Sort                                   (DFT_MON_SORT) = OFF
 WLM dispatcher min. utilization (%) (WLM_DISP_MIN_UTIL) = 5
 WLM dispatcher min. utilization (%) (WLM_DISP_MIN_util) = 5


 Communication buffer exit library list (COMM_EXIT_LIST) =
 Current effective arch level         (CUR_EFF_ARCH_LVL) = V:11 R:5 M:9 F:0 I:0 SB:0
 Current effective code level         (CUR_EFF_CODE_LVL) = V:11 R:5 M:9 F:0 I:0 SB:0

 Keystore type                           (KEYSTORE_TYPE) = NONE
 Keystore location                   (KEYSTORE_LOCATION) =

 Path to python runtime                    (PYTHON_PATH) =
 Path to R runtime                              (R_PATH) =

 Multipart upload part size            (MULTIPARTSIZEMB) = 100
""".strip()

DB2_MANAGER_CONFIGURATION_EMPTY = ""

DB2_MANAGER_CONFIGURATION_INCORRECT = """

          Database Manager Configuration

""".strip()

db2_databases_configuration_path = "insights_commands/runuser_-l_dbp_-c_db2_get_database_configuration_for_TESTD1"
db2_database_manager_path = "insights_commands/runuser_-l_dbp_-c_db2_-c_db2_get_dbm_cfg"


def test_db2ls():
    db2ls = Db2ls(context_wrap(DB2LS_A_C))
    assert len(db2ls) == 2
    assert db2ls[0]['PATH'] == '/opt/ibm/db2/V11.5'
    assert db2ls[0]['VRMF'] == '11.5.6.0'
    assert db2ls[0]['FIXPACK'] == '0'
    assert db2ls[0]['SPECIAL'] == ''
    assert db2ls[0]['INSTALLTIME'] == 'Fri Jan 14 20:20:07 2022 CST'
    assert db2ls[0]['INSTALLERUID'] == '0'
    assert db2ls[1]['PATH'] == '/opt/ibm/db2/V11.5_01'
    assert db2ls[1]['VRMF'] == '11.5.7.0'
    assert db2ls[1]['FIXPACK'] == '0'
    assert db2ls[1]['SPECIAL'] == ''
    assert db2ls[1]['INSTALLTIME'] == 'Fri Feb 11 10:34:51 2022 CST'
    assert db2ls[1]['INSTALLERUID'] == '0'

    with pytest.raises(SkipComponent):
        Db2ls(context_wrap(DB2LS_A_C_EMPTY1))
    with pytest.raises(SkipComponent):
        Db2ls(context_wrap(DB2LS_A_C_EMPTY2))


def test_db2_databases_configuration():
    db2_configuration_user = db2.Db2DatabaseConfiguration(context_wrap(DB2_CONFIGURATION_USER, path=db2_databases_configuration_path))
    assert db2_configuration_user["Database configuration release level"] == "0x1500"
    assert db2_configuration_user["All committed transactions have been written to disk"] == "YES"
    assert db2_configuration_user["SORTHEAP size to be used by optimizer   (opt_SORTHEAP)"] == "0"
    assert db2_configuration_user["user"] == "dbp"
    assert db2_configuration_user["db_name"] == "TESTD1"

    with pytest.raises(SkipComponent) as e:
        db2.Db2DatabaseConfiguration(context_wrap(DB2_CONFIGURATION_USER_EMPTY, path=db2_databases_configuration_path))
    assert "The result is empty" in str(e)

    with pytest.raises(SkipComponent) as e:
        db2.Db2DatabaseConfiguration(context_wrap(DB2_CONFIGURATION_USER_INCORRECT, path=db2_databases_configuration_path))
    assert "The format is incorrect" in str(e)


def test_db2_database_manager():
    db2_manager_configuration = db2.Db2DatabaseManager(context_wrap(DB2_MANAGER_CONFIGURATION, path=db2_database_manager_path))
    assert db2_manager_configuration["NUMDB"] == "32"
    assert db2_manager_configuration["Database manager configuration release level"] == "0x1500"
    assert db2_manager_configuration["WLM dispatcher min. utilization (%) (WLM_DISP_MIN_util)"] == "5"
    assert db2_manager_configuration["user"] == "dbp"

    with pytest.raises(SkipComponent) as e:
        db2.Db2DatabaseManager(context_wrap(DB2_MANAGER_CONFIGURATION_EMPTY, path=db2_database_manager_path))
    assert "The result is empty" in str(e)

    with pytest.raises(SkipComponent) as e:
        db2.Db2DatabaseManager(context_wrap(DB2_MANAGER_CONFIGURATION_INCORRECT, path=db2_database_manager_path))
    assert "The format is incorrect" in str(e)


def test_doc_examples():
    env = {
        'db2databaseconfiguration': db2.Db2DatabaseConfiguration(context_wrap(DB2_CONFIGURATION_USER, path=db2_databases_configuration_path)),
        'db2databasemanager': db2.Db2DatabaseManager(context_wrap(DB2_MANAGER_CONFIGURATION, path=db2_database_manager_path)),
        'db2ls': Db2ls(context_wrap(DB2LS_A_C))
    }
    failed, total = doctest.testmod(db2, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import sapcontrol
from insights.parsers.sapcontrol import SAPControlSystemUpdateList
from insights.tests import context_wrap

RKS_STATUS = """
29.01.2019 01:20:36
GetSystemUpdateList
OK
hostname, instanceNr, status, starttime, endtime, dispstatus
vm37-39, 00, Running, 29.01.2019 00:00:02, 29.01.2019 01:10:11, GREEN
vm37-39, 02, Running, 29.01.2019 00:00:05, 29.01.2019 01:11:11, GREEN
vm37-39, 03, Running, 29.01.2019 00:00:05, 29.01.2019 01:12:36, GREEN
""".strip()

RKS_STATUS_AB1 = """
""".strip()

RKS_STATUS_AB2 = """
29.01.2019 01:20:26
GetSystemUpdateList
FAIL: NIECONN_REFUSED (Connection refused), NiRawConnect failed in plugin_fopen()
""".strip()

RKS_STATUS_AB3 = """
29.01.2019 01:20:36
GetSystemUpdateList
OK
hostname, instanceNr, status, starttime, endtime, dispstatus
""".strip()


def test_sapcontrol_rks_abnormal():
    with pytest.raises(SkipComponent):
        SAPControlSystemUpdateList(context_wrap(RKS_STATUS_AB1))
    with pytest.raises(ParseException):
        SAPControlSystemUpdateList(context_wrap(RKS_STATUS_AB2))
    with pytest.raises(SkipComponent):
        SAPControlSystemUpdateList(context_wrap(RKS_STATUS_AB3))


def test_sapcontrol_status():
    rks = SAPControlSystemUpdateList(context_wrap(RKS_STATUS))
    assert rks.is_running
    assert rks.is_green
    assert rks.data[0]['status'] == "Running"
    assert rks.data[1]['instanceNr'] == "02"
    assert rks.data[-1]['dispstatus'] == "GREEN"


def test_doc_examples():
    env = {
            'rks': SAPControlSystemUpdateList(context_wrap(RKS_STATUS)),
          }
    failed, total = doctest.testmod(sapcontrol, globs=env)
    assert failed == 0



import doctest

from insights.parsers import virtlogd_conf
from insights.tests import context_wrap


VIRTLOGD_CONF = '''
# Master virtlogd daemon configuration file
#

#################################################################
#
# Logging controls
#

# Logging level: 4 errors, 3 warnings, 2 information, 1 debug
# basically 1 will log everything possible
#log_level = 3

# Logging filters:
# A filter allows to select a different logging level for a given category
# of logs
# The format for a filter is one of:
#    x:name
#    x:+name
#      where name is a string which is matched against source file name,
#      e.g., "remote", "qemu", or "util/json", the optional "+" prefix
#      tells libvirt to log stack trace for each message matching name,
#      and x is the minimal level where matching messages should be logged:
#    1: DEBUG
#    2: INFO
#    3: WARNING
#    4: ERROR
#
# Multiple filter can be defined in a single @filters, they just need to be
# separated by spaces.
#
# e.g. to only get warning or errors from the remote layer and only errors
# from the event layer:
#log_filters="3:remote 4:event"

# Logging outputs:
# An output is one of the places to save logging information
# The format for an output can be:
#    x:stderr
#      output goes to stderr
#    x:syslog:name
#      use syslog for the output and use the given name as the ident
#    x:file:file_path
#      output to a file, with the given filepath
#    x:journald
#      ouput to the systemd journal
# In all case the x prefix is the minimal level, acting as a filter
#    1: DEBUG
#    2: INFO
#    3: WARNING
#    4: ERROR
#
# Multiple output can be defined, they just need to be separated by spaces.
# e.g. to log all warnings and errors to syslog under the virtlogd ident:
#log_outputs="3:syslog:virtlogd"
#

# The maximum number of concurrent client connections to allow
# over all sockets combined.
#max_clients = 1024


# Maximum file size before rolling over. Defaults to 2 MB
#max_size = 2097152

# Maximum number of backup files to keep. Defaults to 3,
# not including the primary active file
max_backups = 3
'''


def test_virtlogd_conf():
    conf = virtlogd_conf.VirtlogdConf(context_wrap(VIRTLOGD_CONF, path='/etc/libvirt/virtlogd.conf'))
    assert conf.get('max_backups') == '3'

    conf = virtlogd_conf.VirtlogdConf(context_wrap(VIRTLOGD_CONF.replace("#max_size = 2097152", "max_size = 12582912")))
    max_size = conf.get('max_size', None)
    assert int(max_size) == 12582912


def test_virtlogd_conf_documentation():
    failed_count, tests = doctest.testmod(
        virtlogd_conf,
        globs={'conf': virtlogd_conf.VirtlogdConf(context_wrap(VIRTLOGD_CONF))}
    )
    assert failed_count == 0



from mock.mock import patch

from insights.client.config import InsightsConfig
from insights.client.core_collector import CoreCollector


@patch('insights.client.core_collector.logger')
@patch('insights.client.core_collector.systemd_notify_init_thread', return_value=None)
@patch('insights.client.core_collector.InsightsArchive.create_archive_dir', return_value=None)
@patch('insights.client.core_collector.collect', return_value=None)
def test_run_collection(collect, create_archive_dir, systemd_notify_init_thread, logger):
    conf = InsightsConfig()
    cc = CoreCollector(conf)
    cc.run_collection({})
    systemd_notify_init_thread.assert_called_once()
    create_archive_dir.assert_called_once()
    collect.collect.assert_called_once()
    logger.debug.assert_called_with('Core collection finished.')



from pytest import mark

from insights.cleaner import Cleaner
from insights.client.config import InsightsConfig

test_data = 'test\nabcd\n \n\n1234\npwd: p4ssw0rd\n'.splitlines()


@mark.parametrize("obfuscate", [True, False])
def test_clean_content_patterns_exclude_regex(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    rm_conf = {'patterns': {'regex': ['12.*4', '^abcd']}}
    pp = Cleaner(conf, rm_conf)
    ret = pp.clean_content(test_data, [])
    # content IS changed
    assert test_data != ret
    assert '1234' not in ret
    assert 'abcd' not in ret


@mark.parametrize("obfuscate", [True, False])
def test_clean_content_result_empty(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    rm_conf = {'patterns': {'regex': [' ', '12.*4', '^abcd', 'test', 'pwd', 'w0rd']}}
    pp = Cleaner(conf, rm_conf)
    ret = pp.clean_content(test_data, [])
    # result content is Empty
    assert len(ret) == 0


@mark.parametrize("obfuscate", [True, False])
def test_clean_content_patterns_exclude_no_regex(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    rm_conf = {'patterns': ['1234', 'abcde']}
    pp = Cleaner(conf, rm_conf)
    ret = pp.clean_content(test_data, [])
    # content IS changed
    assert test_data != ret
    assert '1234' not in ret
    assert 'abcd' in ret


@mark.parametrize("obfuscate", [True, False])
def test_clean_content_patterns_exclude_empty(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    rm_conf = {'patterns': []}
    pp = Cleaner(conf, rm_conf)
    ret = pp.clean_content(test_data, [])
    # file is NOT changed
    assert test_data == ret


@mark.parametrize("obfuscate", [True, False])
def test_clean_content_exclude_none(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    pp = Cleaner(conf, None)
    ret = pp.clean_content(test_data, [])
    # file is NOT changed
    assert test_data == ret


@mark.parametrize(
    ("line", "expected"),
    [
        (
            "what's your name? what day is today?",
            "what's your keyword0? what keyword1 is tokeyword1?",
        ),
    ],
)
@mark.parametrize("obfuscate", [True, False])
def test_clean_content_keyword_extract_cases(obfuscate, line, expected):
    conf = InsightsConfig(obfuscate=obfuscate)
    pp = Cleaner(conf, {'keywords': ['name', 'day']})
    actual = pp.clean_content(line)
    assert actual == expected


@mark.parametrize(
    ("line", "expected"),
    [
        ("test1.abc.com: it's myserver? what is yours?", None),
        ("testabc: it's mykey? what is yours?", None),
        (
            "testabc: it's my1key? what is yours?",
            "testabc: it's my1key? what is yours?",
        ),
    ],
)
def test_clean_content_exclude_patterns(line, expected):
    c = InsightsConfig()
    pp = Cleaner(c, {'patterns': ['myserver', 'mykey']})
    actual = pp.clean_content(line)
    assert actual == expected


@mark.parametrize(
    ("line", "expected"),
    [
        ("test.abc.com: it's myserver? what is yours?", None),
        ("testabc: it's mykey? what is yours?", None),
        ("testabc: it's my1key? what is yours?", None),
        ("test1: it's my-key? what is yours?", "test1: it's my-key? what is yours?"),
    ],
)
def test_clean_content_patterns_regex(line, expected):
    c = InsightsConfig()
    pp = Cleaner(c, {'patterns': {'regex': ['myserver', r'my(\w*)key']}})
    actual = pp.clean_content(line)
    assert actual == expected


@mark.parametrize(
    ("line", "expected"),
    [
        ("test.abc.com: it's myserver? what is yours?", None),
        ("testabc: it's mykey? what is yours?", None),
        ("testabc: it's my1key? what is yours?", None),
        ("test1: it's my-key? what is yours?", None),
        ("test: it's my-key? what is yours?", "test: it's my-key? what is yours?"),
    ],
)
def test_clean_content_patterns_posix_regex(line, expected):
    c = InsightsConfig()
    pp = Cleaner(c, {'patterns': {'regex': ['myserver', r'my(\w*)key', 'test[[:digit:]]']}})
    actual = pp.clean_content(line)
    assert actual == expected



from insights.parsers.sestatus import SEStatus
from insights.tests import context_wrap


SESTATUS_ENFORCING = '''
Loaded policy name:             targeted
Current mode:                   enforcing
Mode from config file:          enforcing
Policy MLS status:              enabled
Policy deny_unknown status:     allowed
Max kernel policy version:      30

Policy booleans:
abrt_anon_write                             off
abrt_handle_event                           off
abrt_upload_watch_anon_write                on
antivirus_can_scan_system                   off
antivirus_use_jit                           off
auditadm_exec_content                       on
'''

SESTATUS_PERMISSIVE = '''
SELinux status:                 enabled
SELinuxfs mount:                /sys/fs/selinux
SELinux root directory:         /etc/selinux
Loaded policy name:             targeted
Current mode:                   permissive
Mode from config file:          permissive
Policy MLS status:              enabled
Policy deny_unknown status:     allowed
Max kernel policy version:      28

Policy booleans:
abrt_anon_write                             off
abrt_handle_event                           off
abrt_upload_watch_anon_write                on
antivirus_can_scan_system                   off
antivirus_use_jit                           off
auditadm_exec_content                       on
authlogin_nsswitch_use_ldap                 on
authlogin_radius                            off
authlogin_yubikey                           off
awstats_purge_apache_log_files              off
'''

SESTATUS_DISABLED = '''
SELinux status:                 disabled
'''


def test_sestatus():
    sestatus_info = SEStatus(context_wrap(SESTATUS_ENFORCING)).data

    assert sestatus_info['loaded_policy_name'] == 'targeted'
    assert sestatus_info['current_mode'] == 'enforcing'
    assert sestatus_info['mode_from_config_file'] == 'enforcing'
    assert sestatus_info['policy_mls_status'] == 'enabled'
    assert sestatus_info['policy_deny_unknown_status'] == 'allowed'
    assert sestatus_info['max_kernel_policy_version'] == '30'
    assert sestatus_info['selinux_status'] == 'enforcing'
    assert sestatus_info['policy_booleans'] == {'abrt_anon_write': False,
                                                'abrt_handle_event': False,
                                                'abrt_upload_watch_anon_write': True,
                                                'antivirus_can_scan_system': False,
                                                'antivirus_use_jit': False,
                                                'auditadm_exec_content': True}
    assert sorted(sestatus_info) == sorted([
        'loaded_policy_name', 'current_mode', 'mode_from_config_file',
        'policy_mls_status', 'policy_deny_unknown_status',
        'max_kernel_policy_version', 'policy_booleans', 'selinux_status'
    ])

    perm_info = SEStatus(context_wrap(SESTATUS_PERMISSIVE))
    assert perm_info is not None
    assert perm_info.data
    assert 'selinux_status' in perm_info.data
    assert perm_info.data['selinux_status'] == 'enabled'
    assert perm_info.data['selinuxfs_mount'] == '/sys/fs/selinux'
    assert perm_info.data['selinux_root_directory'] == '/etc/selinux'
    assert perm_info.data['loaded_policy_name'] == 'targeted'
    assert perm_info.data['current_mode'] == 'permissive'
    assert perm_info.data['mode_from_config_file'] == 'permissive'
    assert perm_info.data['policy_mls_status'] == 'enabled'
    assert perm_info.data['policy_deny_unknown_status'] == 'allowed'
    assert perm_info.data['max_kernel_policy_version'] == '28'
    assert perm_info.data['policy_booleans'] == {
        'abrt_anon_write': False,
        'abrt_handle_event': False,
        'abrt_upload_watch_anon_write': True,
        'antivirus_can_scan_system': False,
        'antivirus_use_jit': False,
        'auditadm_exec_content': True,
        'authlogin_nsswitch_use_ldap': True,
        'authlogin_radius': False,
        'authlogin_yubikey': False,
        'awstats_purge_apache_log_files': False,
    }

    disabled_info = SEStatus(context_wrap(SESTATUS_DISABLED))
    assert disabled_info is not None
    assert disabled_info.data
    assert disabled_info.data['loaded_policy_name'] is None
    assert disabled_info.data['current_mode'] == 'disabled'
    assert disabled_info.data['mode_from_config_file'] == 'disabled'
    assert disabled_info.data['policy_mls_status'] == 'disabled'
    assert disabled_info.data['policy_deny_unknown_status'] == 'disabled'
    assert disabled_info.data['max_kernel_policy_version'] is None
    assert 'selinux_status' in disabled_info.data
    assert disabled_info.data['selinux_status'] == 'disabled'
    assert disabled_info.data['policy_booleans'] == {}
